/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.AggregateLiteral;
import aspmodificator.parser.gringo.ConstTerm;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.MathTerm;
import aspmodificator.parser.gringo.Term;

public class AggregateLiteralModificationRule extends
		AbstractLiteralModificationRule {

	public AggregateLiteralModificationRule() {
		super();
	}

	public AggregateLiteralModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	private void bothInteger(Term lowerConstant, Term upperConstant,
			boolean modifyLower, boolean shouldIncrement) {

		int lower = 1;
		int upper = 1;

		if (lowerConstant != null)
			lower = Integer.parseInt(lowerConstant.getIdentifier());

		if (upperConstant != null)
			upper = Integer.parseInt(upperConstant.getIdentifier());

		boolean sameValue = (lower == upper);
		boolean zeroValue = (lower == 0);

		// security that lower is always greater or equal 0
		if (modifyLower && !shouldIncrement && zeroValue) {
			shouldIncrement = true;
		}

		// security that lower is always less or equal upper
		if (modifyLower && shouldIncrement && sameValue && zeroValue) {
			modifyLower = false;
		}

		if (modifyLower) {
			if (sameValue || !shouldIncrement) {
				lowerConstant.setIdentifier(Integer.toString(lower - 1));
				if (writeComments) {
					lowerConstant
							.addSucceedingComment("%* ModificationNote: lower value has been decreased. Original value was "
									+ lower + "  *%");
				}
			} else {
				lowerConstant.setIdentifier(Integer.toString(lower + 1));
				if (writeComments) {
					lowerConstant
							.addSucceedingComment("%* ModificationNote: lower value has been increased. Original value was "
									+ lower + "  *%");
				}
			}
		} else {
			if (sameValue || shouldIncrement) {
				upperConstant.setIdentifier(Integer.toString(upper + 1));
				if (writeComments) {
					lowerConstant
							.addSucceedingComment("%* ModificationNote: upper value has been increased. Original value was "
									+ upper + "  *%");
				}
			} else {
				upperConstant.setIdentifier(Integer.toString(upper - 1));
				if (writeComments) {
					lowerConstant
							.addSucceedingComment("%* ModificationNote: upper value has been decreased. Original value was "
									+ upper + "  *%");
				}
			}
		}
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					for (Literal lit : rule.getHead()) {
						if (lit.isModified())
							continue;

						List<Literal> tmpList = Literal.findClassesByType(
								AggregateLiteral.class, lit);

						for (Literal mt : tmpList) {
							AggregateLiteral aggLit = (AggregateLiteral) mt;

							if (!aggLit.isModified()
									&& (aggLit.getLower() != null || aggLit
											.getUpper() != null))
								result.add(mt);
						}
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					for (Literal lit : rule.getBody()) {
						if (lit.isModified())
							continue;

						List<Literal> tmpList = Literal.findClassesByType(
								AggregateLiteral.class, lit);

						for (Literal mt : tmpList) {
							AggregateLiteral aggLit = (AggregateLiteral) mt;

							if (!aggLit.isModified()
									&& (aggLit.getLower() != null || aggLit
											.getUpper() != null))
								result.add(mt);
						}
					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.ATA;
	}

	@Override
	protected void modifyLiteral(Literal lit) {
		AggregateLiteral aggregateLiteral = (AggregateLiteral) lit;
		Term lowerConstant = aggregateLiteral.getLower();

		Term upperConstant = aggregateLiteral.getUpper();

		boolean modifyLower = random.nextBoolean();
		boolean shouldIncrement = random.nextBoolean();

		if (lowerConstant == null)
			modifyLower = false;

		if (upperConstant == null)
			modifyLower = true;

		if ((lowerConstant == null || ConstTerm.class
				.isAssignableFrom(lowerConstant.getClass()))
				&& (upperConstant == null || ConstTerm.class
						.isAssignableFrom(upperConstant.getClass()))) {
			bothInteger(lowerConstant, upperConstant, modifyLower,
					shouldIncrement);
		} else {

			if (ConstTerm.class.isAssignableFrom(lowerConstant.getClass())
					&& ConstTerm.class.isAssignableFrom(upperConstant
							.getClass())) {
				if (lowerConstant.toString().equals(upperConstant.toString())) {
					if (shouldIncrement && modifyLower)
						shouldIncrement = false;

					if (!shouldIncrement && !modifyLower)
						shouldIncrement = true;
				}
			}

			if (modifyLower) {

				Term newLiteral = toRename(lit, aggregateLiteral,
						lowerConstant, shouldIncrement, "lower");
				aggregateLiteral.setLower(newLiteral);
			} else {
				Term newLiteral = toRename(lit, aggregateLiteral,
						upperConstant, shouldIncrement, "upper");
				aggregateLiteral.setUpper(newLiteral);
			}

		}

	}

	private Term toRename(Literal lit, AggregateLiteral aggregateLiteral,
			Term target, boolean shouldIncrement, String targetName) {

		Term returnValue = target;
		if (ConstTerm.class.isAssignableFrom(target.getClass())) {
			int value = Integer.parseInt(target.getIdentifier());
			if (shouldIncrement || value <= 1) {
				target.setIdentifier(Integer.toString(value + 1));
				if (writeComments) {
					target.addSucceedingComment("%* ModificationNote: "
							+ targetName
							+ " value has been incremented. Original value was "
							+ value + "  *%");
				}
			} else {
				target.setIdentifier(Integer.toString(value - 1));
				if (writeComments) {
					target.addSucceedingComment("%* ModificationNote: "
							+ targetName
							+ " value has been decremented. Original value was "
							+ value + "  *%");
				}
			}
		} else {
			if (shouldIncrement) {
				returnValue = new MathTerm(MathTerm.PLUS, target,
						new ConstTerm(Integer.valueOf(1), lit.getLineNo()),
						lit.getLineNo());
				if (writeComments) {
					returnValue
							.addSucceedingComment("%* ModificationNote: "
									+ targetName
									+ " value has been incremented. Original value was "
									+ target.toString() + "  *%");
				}
			} else {
				returnValue = new MathTerm(MathTerm.MINUS, target,
						new ConstTerm(Integer.valueOf(1), lit.getLineNo()),
						lit.getLineNo());
				if (writeComments) {
					returnValue
							.addSucceedingComment("%* ModificationNote: "
									+ targetName
									+ " value has been decremented. Original value was "
									+ target.toString() + "  *%");
				}
			}
		}
		return returnValue;
	}

}
