/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Term;
import aspmodificator.parser.gringo.VarTerm;

import common.Pair;
import common.Tripple;

public class RenameVariableModificationRule extends
		AbstractReplacementLiteralModificationRule<String> {

	protected static final String COMMENT_RENAME_VARIABLE = "%*ModificationNote: variable has been renamed*%";

	public RenameVariableModificationRule() {
		super();
	}

	public RenameVariableModificationRule(int minApplication, int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	protected List<Tripple<Literal, Literal, List<String>>> findApplicableObjects(
			List<LineBase> program) {
		List<Tripple<Literal, Literal, List<String>>> result = new ArrayList<Tripple<Literal, Literal, List<String>>>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				// collect all variables
				Set<String> variables = new HashSet<String>();
				if (rule.getHead() != null && !rule.getHead().isEmpty()) {
					for (Literal lit : rule.getHead()) {
						List<VarTerm> vars = lit.getVariables();
						if (vars != null && vars.size() > 0) {
							for (VarTerm var : vars)
								variables.add(var.getIdentifier());
						}
					}
				}

				if (rule.getBody() != null && !rule.getBody().isEmpty()) {
					for (Literal lit : rule.getBody()) {
						List<VarTerm> vars = lit.getVariables();
						if (vars != null && vars.size() > 0) {
							for (VarTerm var : vars) {
								variables.add(var.getIdentifier());
							}
						}
					}
				}

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {

					List<Pair<Literal, Literal>> litVars = Literal
							.findClassesByTypeWithParent(VarTerm.class,
									rule.getHead(), null);

					for (Pair<Literal, Literal> pair : litVars) {

						VarTerm var = (VarTerm) pair.getFirst();

						// test if this variable can be replaced and if so,
						// with which values
						String oldIdentifier = var.getIdentifier(); // save
																	// original
																	// identifier

						List<String> values = new ArrayList<String>();
						for (String otherVar : variables) {
							if (oldIdentifier.equals(otherVar))
								continue;

							var.setIdentifier(otherVar);

							// test if replacing var with otherVar
							// results in a safe rule
							if (GringoRuleUtil.checkSafety(rule))
								values.add(otherVar);
						}

						// restore identifier
						var.setIdentifier(oldIdentifier);

						// save variable with possible replacements
						if (values.size() > 0)
							result.add(new Tripple<Literal, Literal, List<String>>(
									var, pair.getSecond(), values));
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {

					List<Pair<Literal, Literal>> litVars = Literal
							.findClassesByTypeWithParent(VarTerm.class,
									rule.getBody(), null);

					for (Pair<Literal, Literal> pair : litVars) {

						VarTerm var = (VarTerm) pair.getFirst();

						// test if this variable can be replaced and if so,
						// with which values
						String oldIdentifier = var.getIdentifier(); // save
																	// original
																	// identifier

						List<String> values = new ArrayList<String>();
						for (String otherVar : variables) {
							if (oldIdentifier != null && otherVar != null
									&& oldIdentifier.equals(otherVar))
								continue;

							var.setIdentifier(otherVar);

							// test if replacing var with otherVar
							// results in a safe rule
							if (GringoRuleUtil.checkSafety(rule))
								values.add(otherVar);
						}

						// restore identifier
						var.setIdentifier(oldIdentifier);

						// save variable with possible replacements
						if (values.size() > 0)
							result.add(new Tripple<Literal, Literal, List<String>>(
									var, pair.getSecond(), values));

					}
				}
			}
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TRV;
	}

	@Override
	protected Literal modifyLiteral(
			Tripple<Literal, Literal, List<String>> tripple, int index) {

		Literal newLit = new Term(tripple.getThird().get(index), tripple
				.getFirst().getLineNo());
		if (writeComments) {
			newLit.addSucceedingComment(COMMENT_RENAME_VARIABLE);
		}

		return newLit;
	}

}
