/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Predicate;

import common.Pair;

public class ReplaceLiteralModificationRule extends AbstractModificationRule {

	public ReplaceLiteralModificationRule() {
		super();
	}

	public ReplaceLiteralModificationRule(int minApplication, int maxApplication) {
		super(minApplication, maxApplication);
	}

	public int apply(List<LineBase> program) {
		// find applicable rules/objects
		List<Pair<Literal, List<Literal>>> applicableObjectPairs = findApplicableObjects(program);

		if (0 == applicableObjectPairs.size())
			return 0;
		int ruleApplied = 0;
		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableObjectPairs.size());

		int applicableObjectNo = applicableObjectPairs.size();

		while (applicableObjectNo > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			applicableObjectNo = applicableObjectPairs.size();
			int objectIndex = -1;

			for (Pair<Literal, List<Literal>> pair : applicableObjectPairs) {

				objectIndex++;

				if (pair.getFirst().isModified()) {
					applicableObjectNo--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(ModificationRuleTypeEnum.LRP, pair
								.getFirst().getLineNo(), objectIndex)) {
					applicableObjectNo--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {
					modifyLiteral(pair);

					ruleApplied++;

					applicableObjectNo--;

					if (useMemory)
						tabuList.setTabu(ModificationRuleTypeEnum.LRP, pair
								.getFirst().getLineNo(), objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		}

		return ruleApplied;
	}

	protected List<Pair<Literal, List<Literal>>> findApplicableObjects(
			List<LineBase> program) {
		List<Pair<Literal, List<Literal>>> result = new ArrayList<Pair<Literal, List<Literal>>>();
		HashSet<Literal> predicates = new HashSet<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null) {
					for (Literal lit : rule.getHead()) {
						if (lit.isModified())
							continue;

						List<Literal> tmpList = Literal.findClassesByType(
								Predicate.class, lit);

						for (Literal mt : tmpList) {

							// this rule only deals with grounded literals - so
							// ignore predicates with variables
							if (mt.getVariables() != null)
								continue;

							predicates.add(mt);

							if (!mt.isModified()
									&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD))
								result.add(new Pair<Literal, List<Literal>>(mt,
										new ArrayList<Literal>()));
						}
					}
				}

				if (rule.getBody() != null) {
					for (Literal lit : rule.getBody()) {
						if (lit.isModified())
							continue;

						List<Literal> tmpList = Literal.findClassesByType(
								Predicate.class, lit);

						for (Literal mt : tmpList) {

							// this rule only deals with grounded literals - so
							// ignore predicates with variables
							if (mt.getVariables() != null)
								continue;

							predicates.add(mt);

							if (!mt.isModified()
									&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY))
								result.add(new Pair<Literal, List<Literal>>(mt,
										new ArrayList<Literal>()));
						}
					}
				}
			}
		}

		removeDuplicates(predicates);
		if (predicates.size() > 1) {
			for (Pair<Literal, List<Literal>> pair : result) {
				Literal removed = removeEqual(predicates, pair.getFirst());
				pair.getSecond().addAll(predicates);

				Collections.sort(pair.getSecond(), new Comparator<Literal>() {

					@Override
					public int compare(Literal o1, Literal o2) {
						return o1.toString().compareTo(o2.toString());
					}
				});
				if (removed != null)
					predicates.add(removed);
			}
		} else {
			result.clear();
		}
		return result;

	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TODO;
	}

	protected void modifyLiteral(Pair<Literal, List<Literal>> pair) {
		Literal predicate = (Literal) pair.getFirst();

		if (writeComments) {
			predicate
					.addSucceedingComment("%* ModificationNote: predicate has been replaced. Original name was \""
							+ predicate.toString() + "\" *%");
		}

		int index = -1;

		// choose the predicate which should be used instead
		index = random.nextInt(pair.getSecond().size());

		Literal otherPredicate = pair.getSecond().get(index);

		// change the identifier of the old predicate to the new one
		predicate.setIdentifier(otherPredicate.getIdentifier());

		// remove terms of the predicate
		if (predicate.getTerms() != null)
			predicate.getTerms().clear();

		// add new terms, it the other predicate has terms
		if (otherPredicate.getTerms() != null) {
			if (predicate.getTerms() == null)
				predicate.setTerms(new ArrayList<Literal>());
			for (Literal term : otherPredicate.getTerms()) {
				try {
					predicate.getTerms().add((Literal) term.clone());
				} catch (CloneNotSupportedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

	}

	private void removeDuplicates(HashSet<Literal> predicates) {
		ArrayList<Literal> toRemoveList = new ArrayList<Literal>();
		for (Literal lit : predicates) {
			if (toRemoveList.contains(lit))
				continue;

			for (Literal lit2 : predicates) {
				if (lit == lit2)
					continue;

				if (lit.toString().equals(lit2.toString()))
					toRemoveList.add(lit2);
			}
		}

		predicates.removeAll(toRemoveList);
	}

	private Literal removeEqual(HashSet<Literal> predicates, Literal first) {
		for (Literal lit : predicates) {
			if (lit.toString().equals(first.toString())) {
				predicates.remove(lit);
				return lit;
			}
		}
		return null;
	}

}
