/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Predicate;

public class SwapStrongNegationModificationRule extends
		AbstractLiteralModificationRule {

	public static final String COMMENT_ADDED = "%* ModificationNote: added strong-negation \"-\" *%";
	public static final String COMMENT_REMOVED = "%* ModificationNote: removed strong-negation \"-\" *%";

	protected boolean requireExistingObject = false;

	public SwapStrongNegationModificationRule() {
		super();
	}

	public SwapStrongNegationModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	public SwapStrongNegationModificationRule(int minApplication,
			int maxApplication, boolean requireExistingPredicate) {
		super(minApplication, maxApplication);
		this.requireExistingObject = requireExistingPredicate;
	}

	@Override
	protected List<Literal> findApplicableObjects(List<LineBase> program) {
		List<Literal> result = new ArrayList<Literal>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null) {
					for (Literal lit : rule.getHead()) {
						if (Predicate.class.isAssignableFrom(lit.getClass())
								&& !lit.isModified()
								&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD))
							result.add(lit);
					}
				}

				if (rule.getBody() != null) {
					for (Literal lit : rule.getBody()) {
						if (Predicate.class.isAssignableFrom(lit.getClass())
								&& !lit.isModified()
								&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY))
							result.add(lit);
					}
				}
			}
		}

		if (requireExistingObject) {
			// find for each literal a similar one with/without strong negation
			// - otherwise remove it from the result list

			ArrayList<Literal> toRemove = new ArrayList<Literal>();

			for (Literal lit : result) {
				boolean found = false;
				for (Literal lit2 : result) {
					if (lit == lit2)
						continue;

					// predicates with different arity are not representing the
					// same predicate - so continue
					if (lit.getArity() != lit2.getArity())
						continue;

					Predicate pred = (Predicate) lit;
					Predicate pred2 = (Predicate) lit2;

					if (pred.isSigned() == pred2.isSigned())
						continue;

					// if their identifier is unequal then they can not be the
					// same literal
					if (!pred.getIdentifier().equals(pred2.getIdentifier()))
						continue;

					found = true;
					break;
				}

				if (!found)
					toRemove.add(lit);
			}

			result.removeAll(toRemove);
		}

		return result;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.SSN;
	}

	public boolean isRequireExistingObject() {
		return requireExistingObject;
	}

	@Override
	protected void modifyLiteral(Literal lit) {

		Predicate pred = (Predicate) lit;
		pred.setNegated(!pred.isNegated());

		if (writeComments) {
			if (pred.isNegated())
				lit.addSucceedingComment(COMMENT_ADDED);
			else
				lit.addSucceedingComment(COMMENT_REMOVED);
		}
	}

	public void setRequireExistingObject(boolean requireExistingObject) {
		this.requireExistingObject = requireExistingObject;
	}

}
