/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.regex.Pattern;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.RelLit;
import aspmodificator.parser.gringo.Term;
import aspmodificator.parser.gringo.VarTerm;

import common.Pair;

public class VariableToConstantModificationRule extends
		AbstractComplexLiteralModificationRule {

	protected static final String COMMENT_VARIABLE_TO_CONSTANT = "%*ModificationNote: term has been modified from a variable to a constant*%";
	protected RuleTargetEnum existenceCheckTarget = RuleTargetEnum.NONE;

	public VariableToConstantModificationRule() {
		super();
	}

	public VariableToConstantModificationRule(int minApplication,
			int maxApplication) {
		super(minApplication, maxApplication);
	}

	@Override
	public int apply(List<LineBase> program) {
		ArrayList<String> constantNames = null;

		if (existenceCheckTarget != RuleTargetEnum.NONE) {
			HashSet<String> constTermList = new HashSet<String>();

			// find constants of the program
			for (LineBase line : program) {
				if (GringoRule.class.isAssignableFrom(line.getClass())) {
					GringoRule rule = (GringoRule) line;

					if (rule.getHead() != null) {
						for (Literal lit : rule.getHead()) {
							List<Literal> tmpList = Literal.findClassesByType(
									Term.class, lit);

							for (Literal l : tmpList)
								constTermList.add(l.getIdentifier());

						}
					}

					if (rule.getBody() != null) {
						for (Literal lit : rule.getBody()) {
							List<Literal> tmpList = Literal.findClassesByType(
									Term.class, lit);

							for (Literal l : tmpList)
								constTermList.add(l.getIdentifier());
						}
					}
				}
			}

			if (constTermList.isEmpty()
					&& existenceCheckTarget == RuleTargetEnum.ALL)
				return 0;
			else
				constantNames = new ArrayList<String>(constTermList);

		}

		// find applicable rules/objects
		List<Pair<Literal, Literal>> applicableObjectPairs = new ArrayList<Pair<Literal, Literal>>();

		// save original
		RuleTargetEnum originalTarget = mrRuleTarget;

		if (originalTarget == RuleTargetEnum.ALL
				|| originalTarget == RuleTargetEnum.HEAD) {
			// set ruleTarget to head to get constant literals from the head of
			// the rules
			mrRuleTarget = RuleTargetEnum.HEAD;
			applicableObjectPairs.addAll(findApplicableObjects(program));
		}

		int noHeadConstants = applicableObjectPairs.size();

		if (originalTarget == RuleTargetEnum.ALL
				|| originalTarget == RuleTargetEnum.BODY) {
			// set ruleTarget to head to get constant literals from the body of
			// the rules
			mrRuleTarget = RuleTargetEnum.BODY;
			applicableObjectPairs.addAll(findApplicableObjects(program));
		}

		// restore original value
		mrRuleTarget = originalTarget;

		int applicableObjectNo = applicableObjectPairs.size();

		if (0 == applicableObjectNo)
			return 0;

		int ruleApplied = 0;

		double chance = (double) (minApplication + maxApplication)
				/ (double) (2 * applicableObjectNo);

		while (applicableObjectNo > 0 && ruleApplied < maxApplication
				&& ruleApplied < minApplication) {

			applicableObjectNo = applicableObjectPairs.size();

			int objectIndex = -1;
			for (Pair<Literal, Literal> pair : applicableObjectPairs) {
				objectIndex++;

				if (pair.getFirst().isModified()) {
					applicableObjectNo--;
					continue;
				}

				if (useMemory
						&& tabuList.isTabu(getRuleTypeEnum(), pair.getFirst()
								.getLineNo(), objectIndex)) {
					applicableObjectNo--;
					continue;
				}

				double randomValue = random.nextDouble();

				if (randomValue <= chance) {
					Literal newLiteral;

					if (existenceCheckTarget == RuleTargetEnum.NONE)
						newLiteral = modifyLiteral(pair);
					else if ((objectIndex < noHeadConstants && existenceCheckTarget == RuleTargetEnum.BODY)
							|| (objectIndex >= noHeadConstants && existenceCheckTarget == RuleTargetEnum.HEAD)) {
						String name = pair.getFirst().getIdentifier()
								.substring(0, 1).toLowerCase()
								+ pair.getFirst().getIdentifier().substring(1);

						boolean contains = constantNames.contains(name);

						constantNames.add(name);
						newLiteral = modifyLiteral(pair, constantNames);

						if (!contains)
							constantNames.remove(name);

					} else {
						newLiteral = modifyLiteral(pair, constantNames);
					}

					int index = pair.getSecond().getTerms()
							.indexOf(pair.getFirst());
					if (index >= 0)
						pair.getSecond().getTerms().remove(index);

					if (newLiteral != null) {
						pair.getSecond().getTerms().add(index, newLiteral);
						newLiteral.setModified(true);
					}

					ruleApplied++;

					applicableObjectNo--;

					if (useMemory)
						tabuList.setTabu(getRuleTypeEnum(), pair.getFirst()
								.getLineNo(), objectIndex);

					if (ruleApplied == maxApplication)
						break;
				}
			}
		}

		return ruleApplied;
	}

	private boolean checkParentPreconditions(Literal parent) {
		if (parent.getClass().equals(RelLit.class))
			return false;
		return true;
	}

	private boolean checkPreconditions(Literal lit) {
		VarTerm term = (VarTerm) lit;

		if (term.getTerms() != null && term.getTerms().size() > 0)
			return false;

		if (term.getIdentifier() != null
				&& Pattern.matches("[A-Z].*", term.getIdentifier()))
			return true;

		return false;
	}

	protected List<Pair<Literal, Literal>> findApplicableObjects(
			List<LineBase> program) {
		List<Pair<Literal, Literal>> result = new ArrayList<Pair<Literal, Literal>>();

		for (LineBase line : program) {
			if (GringoRule.class.isAssignableFrom(line.getClass())) {
				GringoRule rule = (GringoRule) line;

				if (rule.getHead() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.HEAD)) {
					List<Pair<Literal, Literal>> termList = Literal
							.findClassesByTypeWithParent(VarTerm.class,
									rule.getHead(), null);

					for (Pair<Literal, Literal> pair : termList) {
						if (checkPreconditions(pair.getFirst())
								&& checkParentPreconditions(pair.getSecond())
								&& !pair.getFirst().isModified()) {
							// check if resulting rule without the variable
							// would be save
							int index = pair.getSecond().getTerms()
									.indexOf(pair.getFirst());

							pair.getSecond().getTerms().remove(index);

							if (GringoRuleUtil.checkSafety(rule))
								result.add(pair);

							pair.getSecond().getTerms()
									.add(index, pair.getFirst());
						}
					}
				}

				if (rule.getBody() != null
						&& (mrRuleTarget == RuleTargetEnum.ALL || mrRuleTarget == RuleTargetEnum.BODY)) {
					List<Pair<Literal, Literal>> termList = Literal
							.findClassesByTypeWithParent(VarTerm.class,
									rule.getBody(), null);

					for (Pair<Literal, Literal> pair : termList) {
						if (checkPreconditions(pair.getFirst())
								&& checkParentPreconditions(pair.getSecond())
								&& !pair.getFirst().isModified()) {
							// check if resulting rule without the variable
							// would be save
							int index = pair.getSecond().getTerms()
									.indexOf(pair.getFirst());

							pair.getSecond().getTerms().remove(index);

							if (GringoRuleUtil.checkSafety(rule))
								result.add(pair);

							pair.getSecond().getTerms()
									.add(index, pair.getFirst());
						}
					}
				}
			}

		}

		return result;
	}

	public RuleTargetEnum getExistenceCheckTarget() {
		return existenceCheckTarget;
	}

	@Override
	public ModificationRuleTypeEnum getRuleTypeEnum() {
		return ModificationRuleTypeEnum.TVC;
	}

	protected Literal modifyLiteral(Pair<Literal, Literal> pair) {
		String name = pair.getFirst().getIdentifier().substring(0, 1)
				.toLowerCase()
				+ pair.getFirst().getIdentifier().substring(1);

		Literal newLit = new Term(name, pair.getFirst().getLineNo());
		if (writeComments) {
			newLit.addSucceedingComment(COMMENT_VARIABLE_TO_CONSTANT);
		}

		return newLit;
	}

	private Literal modifyLiteral(Pair<Literal, Literal> pair,
			List<String> constTermList) {

		String name = constTermList.get(random.nextInt(constTermList.size()));

		Literal newLit = new Term(name, pair.getFirst().getLineNo());
		if (writeComments) {
			newLit.addSucceedingComment(COMMENT_VARIABLE_TO_CONSTANT);
		}

		return newLit;
	}

	public void setExistenceCheckTarget(RuleTargetEnum existenceCheckTarget) {
		this.existenceCheckTarget = existenceCheckTarget;
	}

}
