/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

import aspmodificator.model.LineBase;

public class GringoRule extends LineBase {

	private List<Literal> head;
	private List<Literal> body;
	private CommentableObject centerCommentable;
	private CommentableObject backCommentable;

	public GringoRule(List<Literal> h, List<Literal> b) {
		this.head = h;
		this.body = b;
		centerCommentable = new CommentableObject();
		backCommentable = new CommentableObject();
	}

	public GringoRule(List<Literal> h, List<Literal> b, int lineNo) {
		super(lineNo);
		this.head = h;
		this.body = b;
		centerCommentable = new CommentableObject();
		backCommentable = new CommentableObject();
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		List<Literal> newHead = null;

		if (head != null) {
			newHead = new ArrayList<Literal>();
			for (Literal lit : head) {
				newHead.add((Literal) lit.clone());
			}
		}

		List<Literal> newBody = null;

		if (body != null) {
			newBody = new ArrayList<Literal>();
			for (Literal lit : body) {
				newBody.add((Literal) lit.clone());
			}
		}

		CommentableObject newCenterCommentable = null;
		CommentableObject newBackCommentable = null;

		if (centerCommentable != null)
			newCenterCommentable = (CommentableObject) centerCommentable
					.clone();

		if (backCommentable != null)
			newBackCommentable = (CommentableObject) backCommentable.clone();

		GringoRule tmpRule = new GringoRule(newHead, newBody, startLineNo);
		tmpRule.setCenterCommentable(newCenterCommentable);
		tmpRule.setBackCommentable(newBackCommentable);
		return tmpRule;
	}

	public ICommentable getBackCommentable() {
		return backCommentable;
	}

	public List<Literal> getBody() {
		return body;
	}

	public ICommentable getCenterCommentable() {
		return centerCommentable;
	}

	public List<Literal> getHead() {
		return head;
	}

	private void setBackCommentable(CommentableObject backCommentable) {
		this.backCommentable = backCommentable;
	}

	public void setBody(List<Literal> body) {
		this.body = body;
	}

	private void setCenterCommentable(CommentableObject centerCommentable) {
		this.centerCommentable = centerCommentable;
	}

	public void setHead(List<Literal> head) {
		this.head = head;
	}

	@Override
	public String toOutputString() {

		int currentLineNo = 0;
		Literal lit = null;
		StringBuffer sb = new StringBuffer();

		if (head != null && head.size() > 0) {

			// initialize currentLineNo
			currentLineNo = head.get(0).getLineNo();

			for (int i = 0; i < head.size() - 1; i++) {
				lit = head.get(i);
				while (lit.getLineNo() > currentLineNo) {
					sb.append("\n");
					currentLineNo++;
				}
				sb.append(lit.toString());
				sb.append(" | ");
			}

			// write last element
			lit = head.get(head.size() - 1);
			while (lit.getLineNo() > currentLineNo) {
				sb.append("\n");
				currentLineNo++;
			}
			sb.append(lit.toString());

			if (body != null)
				sb.append(" ");
		}

		if (body != null && body.size() > 0) {

			// initialize currentLineNo
			if (currentLineNo == 0)
				currentLineNo = body.get(0).getLineNo();

			if (centerCommentable != null
					&& centerCommentable.getPrecedingComments() != null
					&& !centerCommentable.getPrecedingComments().isEmpty()) {
				for (BlockComment bc : centerCommentable.getPrecedingComments()) {
					sb.append(bc.toString());
				}
			}

			sb.append(":- ");

			if (centerCommentable != null
					&& centerCommentable.getSucceedingComments() != null
					&& !centerCommentable.getSucceedingComments().isEmpty()) {
				for (BlockComment bc : centerCommentable
						.getSucceedingComments()) {
					sb.append(bc.toString());
				}
			}

			for (int i = 0; i < body.size() - 1; i++) {
				lit = body.get(i);
				while (lit.getLineNo() > currentLineNo) {
					sb.append("\n");
					currentLineNo++;
				}
				sb.append(lit.toString());
				sb.append(", ");
			}

			// write last element
			lit = body.get(body.size() - 1);
			while (lit.getLineNo() > currentLineNo) {
				sb.append("\n");
				currentLineNo++;
			}

			sb.append(lit.toString());
		}

		if (backCommentable != null
				&& backCommentable.getPrecedingComments() != null
				&& !backCommentable.getPrecedingComments().isEmpty()) {
			for (BlockComment bc : backCommentable.getPrecedingComments()) {
				sb.append(bc.toString());
			}
		}

		sb.append(".");

		if (backCommentable != null
				&& backCommentable.getSucceedingComments() != null
				&& !backCommentable.getSucceedingComments().isEmpty()) {
			for (BlockComment bc : backCommentable.getSucceedingComments()) {
				sb.append(bc.toString());
			}
		}

		return sb.toString();
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();

		if (head != null && head.size() > 0) {
			for (int i = 0; i < head.size() - 1; i++) {

				sb.append(head.get(i).toString());
				sb.append(" | ");
			}
			sb.append(head.get(head.size() - 1).toString());

			if (body != null)
				sb.append(" ");
		}

		if (body != null && body.size() > 0) {

			sb.append(":- ");

			for (int i = 0; i < body.size() - 1; i++) {

				sb.append(body.get(i).toString());
				sb.append(", ");
			}
			sb.append(body.get(body.size() - 1).toString());

		}

		sb.append(".");
		return sb.toString();
	}

}
