/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser.gringo;

import java.util.ArrayList;
import java.util.List;

import common.Pair;

public class Literal implements ICommentable {

	public static List<Literal> findClassesByType(Class<?> clazz, Literal lit) {
		List<Literal> result = new ArrayList<Literal>();

		// if (clazz.isAssignableFrom(lit.getClass()))
		if (lit.getClass().equals(clazz))
			result.add(lit);

		if (lit.getTerms() != null && lit.getTerms().size() > 0) {
			for (Literal term : lit.getTerms()) {
				result.addAll(findClassesByType(clazz, term));
			}
		}

		return result;
	}

	public static List<Pair<Literal, Literal>> findClassesByTypeWithParent(
			Class<?> clazz, List<Literal> literalList, Literal parent) {
		List<Pair<Literal, Literal>> result = new ArrayList<Pair<Literal, Literal>>();

		if (literalList == null || literalList.size() == 0)
			return result;

		for (Literal lit : literalList) {
			// if (clazz.isAssignableFrom(lit.getClass()))
			if (clazz.equals(lit.getClass()))
				result.add(new Pair<Literal, Literal>(lit, parent));

			if (lit.getTerms() != null && lit.getTerms().size() > 0) {
				result.addAll(findClassesByTypeWithParent(clazz,
						lit.getTerms(), lit));
			}
		}

		return result;
	}

	/**
	 * default negation (is signed with a 'not')
	 */
	protected boolean signed;

	protected List<Literal> terms;

	protected String identifier;

	protected List<BlockComment> precedingComments;

	protected List<BlockComment> succeedingComments;

	protected boolean modified = false;

	protected int lineNo = -1;

	public Literal() {
	}

	public Literal(Literal lit) {
		this.identifier = lit.identifier;
		this.lineNo = lit.lineNo;
		this.modified = lit.modified;
		this.precedingComments = lit.precedingComments;
		this.signed = lit.signed;
		this.succeedingComments = lit.succeedingComments;
		this.terms = lit.terms;
	}

	protected Literal(String identifier, List<Literal> terms) {
		this.identifier = identifier;
		this.terms = terms;
		this.signed = false;
		this.lineNo = -1;
	}

	public Literal(String id, List<Literal> terms, boolean signed) {
		this.identifier = id;
		this.terms = terms;
		this.signed = signed;
	}

	public Literal(String id, List<Literal> terms, boolean signed, int lineNo) {
		this.identifier = id;
		this.terms = terms;
		this.signed = signed;
		this.lineNo = lineNo;
	}

	public Literal(String identifier, List<Literal> terms, boolean signed,
			List<BlockComment> precedingComments,
			List<BlockComment> succeedingComments) {
		super();
		this.signed = signed;
		this.terms = terms;
		this.identifier = identifier;
		this.precedingComments = precedingComments;
		this.succeedingComments = succeedingComments;
	}

	public Literal(String identifier, List<Literal> terms, boolean signed,
			List<BlockComment> precedingComments,
			List<BlockComment> succeedingComments, int lineNo) {
		super();
		this.signed = signed;
		this.terms = terms;
		this.identifier = identifier;
		this.precedingComments = precedingComments;
		this.succeedingComments = succeedingComments;
		this.lineNo = lineNo;
	}

	protected Literal(String identifier, List<Literal> terms, int lineNo) {
		this.identifier = identifier;
		this.terms = terms;
		this.signed = false;
		this.lineNo = lineNo;
	}

	public void addPrecedingComment(String bc) {
		if (this.precedingComments == null)
			this.precedingComments = new ArrayList<BlockComment>();

		precedingComments.add(0, new BlockComment(bc));
	}

	public void addSucceedingComment(String bc) {
		if (this.succeedingComments == null)
			this.succeedingComments = new ArrayList<BlockComment>();

		succeedingComments.add(0, new BlockComment(bc));
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		List<Literal> newTerms = null;

		if (terms != null) {
			newTerms = new ArrayList<Literal>();
			for (Literal lit : terms) {
				newTerms.add((Literal) lit.clone());
			}
		}

		List<BlockComment> newPrecedingComments = null;

		if (precedingComments != null) {
			newPrecedingComments = new ArrayList<BlockComment>();
			for (BlockComment c : precedingComments) {
				newPrecedingComments.add((BlockComment) c.clone());
			}
		}

		List<BlockComment> newSucceedingComments = null;
		if (succeedingComments != null) {
			newSucceedingComments = new ArrayList<BlockComment>();
			for (BlockComment c : succeedingComments) {
				newSucceedingComments.add((BlockComment) c.clone());
			}
		}
		Literal newLit = null;
		try {
			newLit = getClass().newInstance();

			if (identifier != null)
				newLit.setIdentifier(new String(identifier));
			newLit.setTerms(newTerms);
			newLit.setSigned(signed);
			newLit.setPrecedingComments(newPrecedingComments);
			newLit.setSucceedingComments(newSucceedingComments);
			newLit.setLineNo(lineNo);
		} catch (InstantiationException e) {
			throw new CloneNotSupportedException(e.getMessage());
		} catch (IllegalAccessException e) {
			throw new CloneNotSupportedException(e.getMessage());
		}
		return newLit;

	}

	public boolean equalSignature(Literal lit) {
		if (!this.getClass().equals(lit.getClass()))
			return false;

		if (lit.getIdentifier().equals(getIdentifier())
				&& lit.getArity() == getArity())
			return true;

		return false;
	}

	public int getArity() {
		if (terms == null)
			return 0;

		return terms.size();
	}

	public String getIdentifier() {
		return identifier;
	}

	public int getLineNo() {
		return lineNo;
	}

	public List<BlockComment> getPrecedingComments() {
		return precedingComments;
	}

	public List<BlockComment> getSucceedingComments() {
		return succeedingComments;
	}

	public List<Literal> getTerms() {
		return terms;
	}

	/**
	 * Returns a list with all variables of the literal, or null if no variables
	 * are used.
	 * 
	 * @return (@link List) with (@link VarTerm) variables
	 */
	public List<VarTerm> getVariables() {
		if (terms == null || terms.size() == 0)
			return null;

		List<VarTerm> result = null;
		for (Literal term : terms) {
			if (result == null)
				result = term.getVariables();
			else {
				List<VarTerm> tmp = term.getVariables();
				if (tmp != null)
					result.addAll(tmp);
			}
		}
		return result;
	}

	public boolean isModified() {
		return modified;
	}

	public boolean isSigned() {
		return signed;
	}

	public void setIdentifier(String identifier) {
		this.identifier = identifier;
	}

	private void setLineNo(int lineNo) {
		this.lineNo = lineNo;
	}

	public void setModified(boolean b) {
		this.modified = b;
	}

	public void setPrecedingComments(List<BlockComment> precedingComments) {
		this.precedingComments = precedingComments;
	}

	public void setSigned(boolean signed) {
		this.signed = signed;
	}

	public void setSucceedingComments(List<BlockComment> succeedingComments) {
		this.succeedingComments = succeedingComments;
	}

	public void setTerms(List<Literal> terms) {
		this.terms = terms;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();

		if (precedingComments != null && precedingComments.size() > 0) {
			for (BlockComment bc : precedingComments) {
				sb.append(" ");
				sb.append(bc.getText());
				sb.append(" ");
			}
		}

		if (signed) {
			sb.append("not ");
		}

		if (identifier != null)
			sb.append(identifier);

		if (terms != null && terms.size() > 0) {
			sb.append("(");

			for (int i = 0; i < terms.size() - 1; i++) {
				sb.append(terms.get(i).toString());
				sb.append(", ");
			}

			sb.append(terms.get(terms.size() - 1).toString());
			sb.append(")");
		}

		if (succeedingComments != null && succeedingComments.size() > 0) {
			for (BlockComment bc : succeedingComments) {
				sb.append(" ");
				sb.append(bc.getText());
				sb.append(" ");
			}
		}
		return sb.toString();

	}
}
