/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Term;
import aspmodificator.parser.gringo.VarTerm;

import common.Tripple;

public class ConstantToVariableModificationRuleTest {
	private ConstantToVariableModificationRule rule;
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		rule = new ConstantToVariableModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testAddComments_shouldAddComment_1() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X), a(x).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(true);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		Literal lit = ((GringoRule) program.get(0)).getBody().get(2).getTerms()
				.get(0);

		assertNotNull(lit.getSucceedingComments());
		assertEquals(1, lit.getSucceedingComments().size());
		assertEquals(
				ConstantToVariableModificationRule.COMMENT_CONSTANT_TO_VARIABLE,
				lit.getSucceedingComments().get(0).toString());
	}

	@Test
	public void testAddComments_shouldNotAddComment_1() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X), a(x).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		Literal lit = ((GringoRule) program.get(0)).getBody().get(2).getTerms()
				.get(0);

		assertEquals("X", lit.toString());

		assertNull(lit.getSucceedingComments());
	}

	@Test
	public void testApply_shouldDoNothing_1() throws Exception {
		String code = "a(X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);
		String oldString = program.get(0).toString();
		rule.setWriteComments(true);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());
		assertEquals(oldString, program.get(0).toString());
	}

	@Test
	public void testApply_shouldDoNothing_2() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X), a(f(X)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(f(X))", ((GringoRule) program.get(0)).getBody().get(2)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_1() throws Exception {
		String code = "a(x,Y) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(X,Y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_2() throws Exception {
		String code = "a(x,Y) :- aa(x), b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(X,Y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_3() throws Exception {
		String code = "a(X,Y) :- aa(X), b(x), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("b(X)", ((GringoRule) program.get(0)).getBody().get(1)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_4() throws Exception {
		String code = "a(x,Y) :- aa(x), b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(x,Y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
		assertEquals("aa(X)", ((GringoRule) program.get(0)).getBody().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_5() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X), a(f(x)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(f(X))", ((GringoRule) program.get(0)).getBody().get(2)
				.toString());
	}

	@Test
	public void testApply_shouldTransformTermToVarTerm() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X), a(x).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(2).getTerms()
				.get(0);
		assertEquals(Term.class, lit.getClass());

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		lit = ((GringoRule) program.get(0)).getBody().get(2).getTerms().get(0);
		assertEquals(VarTerm.class, lit.getClass());

	}

	@Test
	public void testFindApplicableObjects_shouldReturn1() throws Exception {
		String code = "a(x,Y,Z) :- Z = X+Y.";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(1, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2() throws Exception {
		String code = "a(x) :- b(x).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(2, pairList.size());
		assertEquals("x", pairList.get(0).getFirst().toString());
		assertEquals("x", pairList.get(1).getFirst().toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_2() throws Exception {
		String code = "a(x,y,Z) :- b(Z).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(2, pairList.size());
		assertEquals("x", pairList.get(0).getFirst().toString());
		assertEquals("y", pairList.get(1).getFirst().toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(X) :- b(X).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}
}
