/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsNull.notNullValue;
import static org.hamcrest.core.IsNull.nullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.CommentableObject;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.ICommentable;
import aspmodificator.parser.gringo.Literal;

import common.Tripple;

public class DeleteTermModificationRuleTest {
	private DeleteTermModificationRule rule;
	private GringoParser parser;

	@Test
	public void bugFixTest_1() throws Exception {
		String code = "digit(X,Y,C) :- char(C,Y), is_digit(C), X = Y-1.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(true);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);
		assertThat(
				program.get(0).toString(),
				is("digit(X,Y,C) :- char(C,Y), %*ModificationNote: Literal is_digit(C) has been removed*% X = Y-1."));
	}

	@Before
	public void setUp() throws Exception {
		rule = new DeleteTermModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testApply_shouldAddComment_1() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y, num(Z), num(X), num(Y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(true);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(1);
		assertThat(lit.getPrecedingComments(), nullValue());
		int res = rule.apply(program);

		assertThat(res, is(1));
		assertThat(lit.getPrecedingComments(), notNullValue());

		assertThat(lit.getPrecedingComments().size(), is(1));
	}

	@Test
	public void testApply_shouldAddComment_2() throws Exception {
		String code = "a(X,Y,Z) :- achja(X,Y,Z), num(X,Y,Z).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(true);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		GringoRule gingoRule = (GringoRule) program.get(0);
		assertThat(
				((CommentableObject) gingoRule.getBackCommentable())
						.getPrecedingComments(),
				nullValue());
		int res = rule.apply(program);

		assertThat(res, is(1));
		assertThat(
				((CommentableObject) gingoRule.getBackCommentable())
						.getPrecedingComments(),
				notNullValue());
		assertThat(((CommentableObject) gingoRule.getBackCommentable())
				.getPrecedingComments().size(), is(1));
	}

	@Test
	public void testApply_shouldDelete_1() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y, num(Z), num(X), num(Y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);
		assertEquals("a(X,Y,Z) :- Z = X+Y, num(X), num(Y).", program.get(0)
				.toString());
	}

	@Test
	public void testApply_shouldDelete_2() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y, num(Z), num(X), num(Y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMrRuleTarget(RuleTargetEnum.ALL);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);
		assertEquals(":- Z = X+Y, num(Z), num(X), num(Y).", program.get(0)
				.toString());
	}

	@Test
	public void testApply_shouldDelete_3() throws Exception {
		String code = "a(X,Y,Z) :- num(X), Z = X+Y, num(Z),  num(Y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		rule.setMrRuleTarget(RuleTargetEnum.ALL);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);
		assertEquals("a(X,Y,Z) :- num(X), num(Z), num(Y).", program.get(0)
				.toString());
	}

	@Test
	public void testApply_shouldDelete_4() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y, num(Z), num(X), num(Y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);
		assertEquals("a(X,Y,Z) :- num(Z), num(X), num(Y).", program.get(0)
				.toString());
	}

	@Test
	public void testApply_shouldDelete_5() throws Exception {
		String code = "a(X) :- X = #min [ enroll(C) : hours(C,H) = H ], test(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(1, res);
		assertEquals("a(X) :- test(X).", program.get(0).toString());
	}

	@Test
	public void testApply_shouldDeleteBodyLiteral() throws Exception {
		String code = "norm(X) :- p(X), not ab(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMrRuleTarget(RuleTargetEnum.BODY);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertThat(res, is(1));
		assertThat(program.get(0).toString(), is("norm(X) :- p(X)."));
	}

	@Test
	public void testApply_shouldDeleteHeadLiteral() throws Exception {
		String code = "norm(X) :- p(X), not ab(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMrRuleTarget(RuleTargetEnum.HEAD);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertThat(res, is(1));
		assertThat(program.get(0).toString(), is(":- p(X), not ab(X)."));
	}

	@Test
	public void testApply_shouldDoNothing() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(0);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(0, res);
	}

	@Test
	public void testApply_shouldNotDelete_1() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(0);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);

		assertEquals(0, res);
	}

	@Test
	public void testFindApplicableRules_shouldReturnEmptyList()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y.";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnEmptyList_3()
			throws Exception {
		String code = "a(X).%*comment*%";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnEmptyList_4()
			throws Exception {
		String code = "1#count{sol(1,1,1),sol(1,1,2),sol(1,1,3),sol(1,1,4),sol(1,1,5),sol(1,1,6),sol(1,1,7),sol(1,1,8),sol(1,1,9)}1.";
		List<LineBase> program = parser.parse(code);

		rule.setIncludeInsideLiterals(false);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnList_4() throws Exception {
		String code = ":- X = 1 + 2.";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnList_5() throws Exception {
		String code = ":- X = 1 + Y, num(Y).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(1, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnNine() throws Exception {
		String code = "1#count{sol(1,1,1),sol(1,1,2),sol(1,1,3),sol(1,1,4),sol(1,1,5),sol(1,1,6),sol(1,1,7),sol(1,1,8),sol(1,1,9)}1.";
		List<LineBase> program = parser.parse(code);
		rule.setIncludeInsideLiterals(true);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(9, pairList.size());
	}

	@Test
	public void testFindApplicableRules_shouldReturnOneRule() throws Exception {
		String code = "a(X) :- b(X), c(X).";

		rule.setMrRuleTarget(RuleTargetEnum.ALL);
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, List<Literal>, ICommentable>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(3, pairList.size());
		assertEquals("a(X)", pairList.get(0).getFirst().toString());
		assertEquals("b(X)", pairList.get(1).getFirst().toString());
		assertEquals("c(X)", pairList.get(2).getFirst().toString());
	}

}
