/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.junit.Assert.*;

import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.RangeTerm;

public class RangeTermModificationRuleTest {

	private GringoParser parser;
	private RangeTermModificationRule rule;

	@Before
	public void setUp() throws Exception {
		rule = new RangeTermModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testApply_shouldModify_1() throws Exception {
		String code = "a(X,Y, 1..6) :- j(X), k(Y).\n b(J,K):- J < K.\n c(M,N):- num(M), num(N).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(false);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("a(X,Y,1..5)", ((GringoRule) program.get(0)).getHead()
				.get(0).toString());
	}

	@Test
	public void testApply_shouldModify_2() throws Exception {
		String code = "a(X,Y, 1..6) :- j(X), k(Y).\n b(J,K):- J < K.\n c(M,N):- num(M), num(N).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("a(X,Y,2..6)", ((GringoRule) program.get(0)).getHead()
				.get(0).toString());
	}

	@Test
	public void testApply_shouldModify_3() throws Exception {
		String code = "a(X,Y, 1..6) :- j(X), k(Y).\n b(J,K):- J < K.\n c(M,N):- num(M), num(N).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(false);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("a(X,Y,0..6)", ((GringoRule) program.get(0)).getHead()
				.get(0).toString());
	}

	@Test
	public void testApply_shouldModify_4() throws Exception {
		String code = "a(X,Y, 1..6) :- a(X), b(Y), c(1..3).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(false);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("c(0..3)", ((GringoRule) program.get(0)).getBody().get(2)
				.toString());
	}

	@Test
	public void testApply_shouldModify_5() throws Exception {
		String code = "a(X,Y, 1..6).\n :- a(X), b(Y), c(1..3).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("c(2..3)", ((GringoRule) program.get(1)).getBody().get(2)
				.toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_1() throws Exception {
		String code = "a(1..2).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_2() throws Exception {
		String code = ":- a(5..10).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn4() throws Exception {
		String code = "top5num(1..X-4,5..X) :- num(X-4..X), top5(1..5), top(X).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(4, literalList.size());
		assertEquals("1..X-4", literalList.get(0).toString());
		assertEquals("5..X", literalList.get(1).toString());
		assertEquals("X-4..X", literalList.get(2).toString());
		assertEquals("1..5", literalList.get(3).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(0, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(x). \n a(X) :- b(X). \n :- c(x).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(0, literalList.size());
	}

	@Test
	public void testModifyLiteral_shouldDecrementLowerBound_1()
			throws Exception {
		String code = "field(0..1).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("-1..1", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementLowerBound_2()
			throws Exception {
		String code = "field(1..3).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("0..3", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementLowerBound_3()
			throws Exception {
		String code = "field(-4..-3).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("-4-1..-3", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementLowerBound_4()
			throws Exception {
		String code = "field(1..2).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("0..2", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementUpperBound_1()
			throws Exception {
		String code = "field(1..3).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("1..2", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementUpperBound_2()
			throws Exception {
		String code = "field(-3..-1).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("-3..-1-1", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementUpperBound_3()
			throws Exception {
		String code = "field(-3..X-1) :- num(X).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("-3..X-1-1", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldDecrementUpperBound_4()
			throws Exception {
		String code = "field(1..2*X) :- num(X).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("1..2*X-1", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementLowerBound_1()
			throws Exception {
		String code = "field(0..2).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("1..2", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementLowerBound_2()
			throws Exception {
		String code = "field(-4..-1).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("-4+1..-1", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementUpperBound_1()
			throws Exception {
		String code = "field(0..1).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("0..2", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementUpperBound_2()
			throws Exception {
		String code = "field(0..2).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("0..3", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementUpperBound_3()
			throws Exception {
		String code = "field(-1..1).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("-1..2", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementUpperBound_4()
			throws Exception {
		String code = "field(0..1).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(false);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("0..2", term.toString());
	}

	@Test
	public void testModifyLiteral_shouldIncrementUpperBound_5()
			throws Exception {
		String code = "field(0..X+1):-num(X).";
		List<LineBase> program = parser.parse(code);
		RangeTerm term = (RangeTerm) ((GringoRule) program.get(0)).getHead()
				.get(0).getTerms().get(0);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false).thenReturn(true);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(term);
		assertEquals("0..X+1+1", term.toString());
	}

}
