/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.RelLit;

public class RelationOperatorModificationRuleTest {

	private RelationOperatorModificationRule rule = null;
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		rule = new RelationOperatorModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testAddComments_shouldAddComment_1() throws Exception {

		String code = "a(X,Y) :- X > Y.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setWriteComments(true);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);
		RelLit relLit = (RelLit) lit;

		int res = rule.apply(program);
		assertEquals(1, res);

		assertEquals(RelLit.EQUAL, relLit.getIdentifier());

		assertNotNull(lit.getSucceedingComments());
		assertEquals(1, lit.getSucceedingComments().size());
		assertEquals(
				RelationOperatorModificationRule.COMMENT_RELATION_OPERATOR_MODIFICATION
						+ ">" + " *%", lit.getSucceedingComments().get(0)
						.toString());
	}

	@Test
	public void testAddComments_shouldNotAddComment_1() throws Exception {
		String code = "a(X,Y) :- X > Y.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);
		RelLit relLit = (RelLit) lit;

		int res = rule.apply(program);
		assertEquals(1, res);

		assertEquals(RelLit.EQUAL, relLit.getIdentifier());

		assertEquals(RelLit.EQUAL, relLit.getIdentifier());
		assertEquals("X " + RelLit.EQUAL + " Y", relLit.toString());

		assertNull(relLit.getSucceedingComments());
	}

	@Test
	public void testApply_shouldDoNothing() throws Exception {
		String code = "a(X) :- b(X), c(X).";
		List<LineBase> program = parser.parse(code);

		String original = program.get(0).toString();

		Random mockedRandom = mock(Random.class);
		rule.setRandom(mockedRandom);

		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);
		assertEquals(0, res);

		assertEquals(original, program.get(0).toString());
	}

	@Test
	public void testApply_shouldModify_1() throws Exception {
		String code = "a(X,Y) :- X > Y.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0);
		rule.setRandom(mockedRandom);

		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);
		RelLit relLit = (RelLit) lit;

		int res = rule.apply(program);
		assertEquals(1, res);

		assertEquals(RelLit.EQUAL, relLit.getIdentifier());
	}

	@Test
	public void testApply_shouldModify_2() throws Exception {
		String code = "a(X,Y) :- X == Y.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0).thenReturn(3);
		rule.setRandom(mockedRandom);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);
		RelLit relLit = (RelLit) lit;

		int res = rule.apply(program);
		assertEquals(1, res);

		assertEquals(RelLit.LOWER, relLit.getIdentifier());
		assertEquals("X < Y", relLit.toString());
	}

	@Test
	public void testApply_shouldModify_3() throws Exception {
		String code = "a(X,Y) :- X > Y, Y != 1.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(2);
		rule.setRandom(mockedRandom);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);
		RelLit relLit = (RelLit) lit;

		int res = rule.apply(program);
		assertEquals(1, res);

		assertEquals(RelLit.GTHAN, relLit.getIdentifier());
		assertEquals("X >= Y", relLit.toString());
	}

	@Test
	public void testApply_shouldModify_4() throws Exception {
		String code = "a(X,Y) :- X > Y, Y != 1.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0);
		rule.setRandom(mockedRandom);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(1);
		RelLit relLit = (RelLit) lit;

		int res = rule.apply(program);
		assertEquals(1, res);

		assertEquals(RelLit.EQUAL, relLit.getIdentifier());
		assertEquals("Y == 1", relLit.toString());
	}

	@Test
	public void testApply_shouldModify_5() throws Exception {
		String code = ":- succ(X,Y), Y > X.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(2);
		rule.setRandom(mockedRandom);

		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);
		assertThat(res, is(1));
		assertThat(((GringoRule) program.get(0)).toString(),
				is(":- succ(X,Y), Y >= X."));
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_1() throws Exception {
		String code = "a(X,Y) :- X > Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X > Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_2() throws Exception {
		String code = "a(X,Y) :- X >= Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X >= Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_3() throws Exception {
		String code = "a(X,Y) :- X < Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X < Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_4() throws Exception {
		String code = "a(X,Y) :- X <= Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X <= Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_5() throws Exception {
		String code = "a(X,Y) :- X == Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X == Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_6() throws Exception {
		String code = "a(X,Y) :- X != Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X != Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_7() throws Exception {
		String code = "a(X) :- X == 1.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X == 1", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_1() throws Exception {
		String code = "a(X,Y) :- X >= Y, Y >= X.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(2, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn3() throws Exception {
		String code = ":- a(X), a(Y), a(Z), X == Y, X == Z, Y == Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(3, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(0, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(x). \n a(X) :- b(X). \n :- c(x).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(0, literalList.size());
	}

	@Test
	public void testModifyLiteral_shouldModify_1() throws Exception {
		String code = "a(X,Y) :- X == Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(1);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(lit);

		assertEquals(RelationOperatorModificationRule.operators[1],
				lit.getIdentifier());
	}

	@Test
	public void testModifyLiteral_shouldModify_2() throws Exception {
		String code = "a(X,Y) :- X == Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0).thenReturn(2);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(lit);

		assertEquals(RelationOperatorModificationRule.operators[2],
				lit.getIdentifier());
	}

	@Test
	public void testModifyLiteral_shouldModify_3() throws Exception {
		String code = "a(X,Y) :- X > Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(0);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(lit);

		assertEquals(RelationOperatorModificationRule.operators[0],
				lit.getIdentifier());
	}

	@Test
	public void testModifyLiteral_shouldModify_4() throws Exception {
		String code = "a(X,Y) :- X != Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		rule.setAvoidNearlyInequalOperators(true);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextInt(2)).thenReturn(0);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(lit);

		assertEquals(RelationOperatorModificationRule.limitedOperatorList[0],
				lit.getIdentifier());
	}

	@Test
	public void testModifyLiteral_shouldModify_5() throws Exception {
		String code = "a(X,Y) :- X != Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		rule.setAvoidNearlyInequalOperators(false);

		Random mockedRandom = mock(Random.class);
		when(
				mockedRandom
						.nextInt(RelationOperatorModificationRule.operators.length))
				.thenReturn(4);
		rule.setRandom(mockedRandom);

		rule.modifyLiteral(lit);

		assertEquals(RelationOperatorModificationRule.operators[4],
				lit.getIdentifier());
	}

}
