/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;

public class RenamePredicateModificationRuleTest {
	private RenamePredicateModificationRule rule;
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		rule = new RenamePredicateModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testApply_shouldDoNothing_1() throws Exception {
		String code = "a(X) :- X != 1, a(X).";
		List<LineBase> program = parser.parse(code);

		String oldString = program.get(0).toOutputString();

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(0, res);
		assertEquals(oldString, program.get(0).toOutputString());
	}

	@Test
	public void testApply_shouldDoNothing_2() throws Exception {
		String code = "a(X,Y) :- b(X), c(Y,Y,Y).";
		List<LineBase> program = parser.parse(code);

		String oldString = program.get(0).toOutputString();

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextBoolean()).thenReturn(false);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(0, res);
		assertEquals(oldString, program.get(0).toOutputString());
	}

	@Test
	public void testApply_shouldRename_1() throws Exception {
		String code = "a(X,Y) :- X > Y.\n b(J,K):-num(J), num(K).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("b(X,Y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldRename_2() throws Exception {
		String code = "a(X,Y) :- X > Y.\n b(J,K):- J < K.\n c(M,N):- num(M), num(N).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("c(X,Y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldRename_3() throws Exception {
		String code = "a(X,Y) :- j(X), k(Y).\n b(J,K):- J < K.\n c(M,N):- num(M), num(N).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertEquals(1, res);
		assertEquals("num(X)", ((GringoRule) program.get(0)).getBody().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldRename_4() throws Exception {
		String code = "r(X,Y):- e(X,Y). :- e(a,b).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(1.0)
				.thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);

		int res = rule.apply(program);
		assertThat(res, is(1));
		assertThat(((GringoRule) program.get(1)).toString(), is(":- r(a,b)."));
	}
}
