/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.hamcrest.CoreMatchers.not;
import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsNull.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.matchers.JUnitMatchers.hasItem;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;

import common.Tripple;

public class ReplaceConstantModificationRuleTest {

	private ReplaceConstantModificationRule rule;
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		rule = new ReplaceConstantModificationRule();
		parser = new GringoParser();
	}
	
	@Test
	public void testAddComments_shouldAddComment_1() throws Exception {
		String code = "a(X,Y) :- b(x), not c(y), a(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(true);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(program, notNullValue());
		assertThat(program.size(), is(1));

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0).getTerms()
				.get(0);

		assertThat(lit.getSucceedingComments(), notNullValue());
		assertThat(lit.getSucceedingComments().size(), is(1));
	}

	@Test
	public void testAddComments_shouldNotAddComment_1() throws Exception {
		String code = "a(x,y) :- b(X), not c(X), a(x).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertThat(program, notNullValue());
		assertEquals(1, program.size());

		Literal lit = ((GringoRule) program.get(0)).getHead().get(0);

		assertEquals("a(y,y)", lit.toString());

		assertNull(lit.getSucceedingComments());
	}

	@Test
	public void testApply_shouldDoNothing_1() throws Exception {
		String code = "a(X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);
		String oldString = program.get(0).toString();
		rule.setWriteComments(true);
		Random mockedRandom = mock(Random.class);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());
		assertEquals(oldString, program.get(0).toString());
	}

	@Test
	public void testApply_shouldDoNothing_2() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X), a(f(X)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(f(X))", ((GringoRule) program.get(0)).getBody().get(2)
				.toString());
	}

	@Test
	public void testApply_shouldDoNothing_3() throws Exception {
		String code = "a(x,Y) :- b(x), not c(x), a(f(x)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(f(x))", ((GringoRule) program.get(0)).getBody().get(2)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_1() throws Exception {
		String code = "a(x).\n b(y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(program, notNullValue());
		assertThat(program.size(), is(2));

		assertEquals("a(y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_2() throws Exception {
		String code = "a(x).\n b(y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(2, program.size());

		assertEquals("b(x)", ((GringoRule) program.get(1)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_3() throws Exception {
		String code = "a(x):- b(y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_4() throws Exception {
		String code = "a(x):- b(f(y)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("b(f(x))", ((GringoRule) program.get(0)).getBody().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_5() throws Exception {
		String code = "a(x):- b(y).\nc(z).\nd(xx).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(2);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(3, program.size());

		assertThat(((GringoRule) program.get(0)).getHead().get(0).toString(),
				is(not("a(x)")));

	}

	@Test
	public void testApply_shouldTransform_6() throws Exception {
		String code = "a(x):- b(y).\nc(z).\nd(xx).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(1.0)
				.thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(1);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(3, program.size());
		assertThat(((GringoRule) program.get(2)).getHead().get(0).toString(),
				is(not("c(z)")));
	}

	@Test
	public void testApply_shouldTransform_7() throws Exception {
		String code = "a(x,y,z):- b(y).\nc(z).\nd(xx).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(1.0)
				.thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(1);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(3, program.size());

		assertEquals("a(x,y,y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_8() throws Exception {
		String code = "a(1). a(2). a(3). c(a). c(b). res(Y) :- a(X), Y = X + 1.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(1.0)
				.thenReturn(1.0).thenReturn(1.0).thenReturn(1.0)
				.thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(6, program.size());

		assertThat(((GringoRule) program.get(5)).toString(),
				is("res(Y) :- a(X), Y = X+2."));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertEquals(0, trippleListe.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(X) :- b(X).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertEquals(0, trippleListe.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_3()
			throws Exception {
		String code = "a(x) :- b(x).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertEquals(0, trippleListe.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_1() throws Exception {
		String code = "a(x).\nb(y).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertThat(trippleListe.size(), is(2));
		assertThat(trippleListe.get(0).getFirst().toString(), is("x"));
		assertThat(trippleListe.get(1).getFirst().toString(), is("y"));
		assertThat(trippleListe.get(0).getThird(), hasItem("y"));
		assertThat(trippleListe.get(1).getThird(), hasItem("x"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_2() throws Exception {
		String code = "a(x) :- b(y).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertEquals(2, trippleListe.size());
		assertThat(trippleListe.get(0).getFirst().toString(), is("x"));
		assertThat(trippleListe.get(1).getFirst().toString(), is("y"));
		assertThat(trippleListe.get(0).getThird(), hasItem("y"));
		assertThat(trippleListe.get(1).getThird(), hasItem("x"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_3() throws Exception {
		String code = "a(x,y,Z) :- b(Z).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertEquals(2, trippleListe.size());
		assertThat(trippleListe.get(0).getFirst().toString(), is("x"));
		assertThat(trippleListe.get(1).getFirst().toString(), is("y"));
		assertThat(trippleListe.get(0).getThird(), hasItem("y"));
		assertThat(trippleListe.get(1).getThird(), hasItem("x"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_4() throws Exception {
		String code = "a(1) :- b(2).";
		List<LineBase> program = parser.parse(code);
		List<Tripple<Literal, Literal, List<String>>> trippleListe = rule
				.findApplicableObjects(program);

		assertNotNull(trippleListe);
		assertEquals(2, trippleListe.size());
		assertThat(trippleListe.get(0).getFirst().toString(), is("1"));
		assertThat(trippleListe.get(1).getFirst().toString(), is("2"));
		assertThat(trippleListe.get(0).getThird(), hasItem("2"));
		assertThat(trippleListe.get(1).getThird(), hasItem("1"));
	}
}
