/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsNull.notNullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;

import common.Pair;

public class ReplaceLiteralModificationRuleTest {
	private ReplaceLiteralModificationRule rule;
	private GringoParser parser;

	@Before
	public void setUp() throws Exception {
		rule = new ReplaceLiteralModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testAddComments_shouldAddComment_1() throws Exception {
		String code = "a(X) :- b, not c, a(X).\n" + "z :- x, not y.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(true);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(1.0)
				.thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(program, notNullValue());
		assertThat(program.size(), is(2));

		Literal lit = ((GringoRule) program.get(1)).getHead().get(0);

		assertThat(lit.getSucceedingComments(), notNullValue());
		assertThat(lit.getSucceedingComments().size(), is(1));
	}

	@Test
	public void testAddComments_shouldNotAddComment_1() throws Exception {
		String code = "a :- b, not c, d.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(3)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertThat(program, notNullValue());
		assertEquals(1, program.size());

		Literal lit = ((GringoRule) program.get(0)).getHead().get(0);

		assertEquals("b", lit.toString());

		assertNull(lit.getSucceedingComments());
	}

	@Test
	public void testApply_shouldTransform_1() throws Exception {
		String code = "a(x).\n b(y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(program, notNullValue());
		assertThat(program.size(), is(2));

		assertEquals("b(y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_2() throws Exception {
		String code = "a(x).\n b(y).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(2, program.size());

		assertEquals("a(x)", ((GringoRule) program.get(1)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_3() throws Exception {
		String code = "a:- b.\nb  :- c.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(2)).thenReturn(1);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(2, program.size());

		assertEquals("c", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_4() throws Exception {
		String code = "a(x) :- b(f(y)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("b(f(y))", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_5() throws Exception {
		String code = "a(x):- b(f(y)).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(1)).thenReturn(0);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(x)", ((GringoRule) program.get(0)).getBody().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldTransform_6() throws Exception {
		String code = "a(x):- b(y).\n c(z).\n d(xx).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(3)).thenReturn(2);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(3, program.size());

		assertThat(((GringoRule) program.get(0)).getBody().get(0).toString(),
				is("d(xx)"));
	}

	@Test
	public void testApply_shouldTransform_7() throws Exception {
		String code = "a:- b.:- not c. \n d :- e, f, not g.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		when(mockedRandom.nextInt(6)).thenReturn(4);
		rule.setRandom(mockedRandom);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(3, program.size());

		assertThat(((GringoRule) program.get(0)).getBody().get(0).toString(),
				is("c"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(X) :- b(X).";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_3()
			throws Exception {
		String code = "result(A,B,C) :- C = A - B.";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_4()
			throws Exception {
		String code = "a :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_5()
			throws Exception {
		String code = "a.";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(0, pairList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_1() throws Exception {
		String code = "a(x).\nb(y).";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertThat(pairList.size(), is(2));
		assertThat(pairList.get(0).getFirst().toString(), is("a(x)"));
		assertThat(pairList.get(1).getFirst().toString(), is("b(y)"));

	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_2() throws Exception {
		String code = "a :- b.";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(2, pairList.size());
		assertThat(pairList.get(0).getFirst().toString(), is("a"));
		assertThat(pairList.get(1).getFirst().toString(), is("b"));

	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_3() throws Exception {
		String code = "a(x,y,z) :- b(z).";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(2, pairList.size());
		assertThat(pairList.get(0).getFirst().toString(), is("a(x,y,z)"));
		assertThat(pairList.get(1).getFirst().toString(), is("b(z)"));

	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_4() throws Exception {
		String code = "a(1) :- b(2).";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(2, pairList.size());
		assertThat(pairList.get(0).getFirst().toString(), is("a(1)"));
		assertThat(pairList.get(1).getFirst().toString(), is("b(2)"));

	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwo_5() throws Exception {
		String code = "a :- b(X), not c(X).\nb.";
		List<LineBase> program = parser.parse(code);
		List<Pair<Literal, List<Literal>>> pairList = rule
				.findApplicableObjects(program);

		assertNotNull(pairList);
		assertEquals(2, pairList.size());
		assertThat(pairList.get(0).getFirst().toString(), is("a"));
		assertThat(pairList.get(0).getSecond().size(), is(1));
		assertThat(pairList.get(1).getFirst().toString(), is("b"));
		assertThat(pairList.get(1).getSecond().size(), is(1));
	}

}
