/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.junit.Assert.*;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.Predicate;

public class SwapStrongNegationModificationRuleTest {

	private GringoParser parser;
	private SwapStrongNegationModificationRule rule;

	@Before
	public void setUp() throws Exception {
		parser = new GringoParser();
		rule = new SwapStrongNegationModificationRule();
	}

	@Test
	public void testAddComments_shouldAddComment_1() throws Exception {
		String code = "a(X) :- X != 1, b(X), c(X).";
		List<LineBase> program = parser.parse(code);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // c(X)

		Predicate pred = (Predicate) lit;

		assertFalse(pred.isNegated());

		rule.modifyLiteral(lit);

		assertTrue(pred.isNegated());

		assertNotNull(lit.getSucceedingComments());
		assertEquals(1, lit.getSucceedingComments().size());
		assertEquals(SwapStrongNegationModificationRule.COMMENT_ADDED, lit
				.getSucceedingComments().get(0).toString());
	}

	@Test
	public void testAddComments_shouldAddComment_2() throws Exception {
		String code = "a(X) :- X != 1, b(X), - c(X).";
		List<LineBase> program = parser.parse(code);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // -c(X)
		Predicate pred = (Predicate) lit;
		rule.setWriteComments(true);
		assertTrue(pred.isNegated());

		rule.modifyLiteral(lit);

		assertFalse(pred.isNegated());

		assertNotNull(lit.getSucceedingComments());
		assertEquals(1, lit.getSucceedingComments().size());
		assertEquals(SwapStrongNegationModificationRule.COMMENT_REMOVED, lit
				.getSucceedingComments().get(0).toString());
	}

	@Test
	public void testAddComments_shouldNotAddComment() throws Exception {
		String code = "a(X) :- X != 1, b(X), -c(X).";
		List<LineBase> program = parser.parse(code);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // -c(X)

		Predicate pred = (Predicate) lit;
		rule.setWriteComments(false);
		assertTrue(pred.isNegated());

		rule.modifyLiteral(lit);

		assertFalse(pred.isNegated());

		assertNull(lit.getSucceedingComments());
	}

	@Test
	public void testapply_shouldSwitchNegation_1() throws Exception {
		String code = "a(X) :- X != 1, b(X), c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // c(X)
		Predicate pred = (Predicate) lit;
		assertFalse(pred.isNegated());

		int res = rule.apply(program);
		assertEquals(1, res);

		assertTrue(pred.isNegated());
	}

	@Test
	public void testapply_shouldSwitchNegation_2() throws Exception {
		String code = "a(X) :- X != 1, b(X), -c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // -c(X)
		Predicate pred = (Predicate) lit;
		assertTrue(pred.isNegated());

		int res = rule.apply(program);
		assertEquals(1, res);

		assertFalse(pred.isNegated());
	}

	@Test
	public void testapply_shouldSwitchNegation_3() throws Exception {
		String code = "a(X) :- X != 1, b(X), -c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setRandom(new Random(50000));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		Literal lit = ((GringoRule) program.get(0)).getHead().get(0); // a(X)
		Predicate pred = (Predicate) lit;
		assertFalse(pred.isNegated());

		int res = rule.apply(program);
		assertEquals(1, res);

		assertTrue(pred.isNegated());
	}

	@Test
	public void testapply_shouldSwitchNegation_4() throws Exception {
		String code = "-a(X) :- X != 1, b(X), -c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setRandom(new Random(50000));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);
		Literal lit = ((GringoRule) program.get(0)).getHead().get(0); // a(X)
		Predicate pred = (Predicate) lit;
		assertTrue(pred.isNegated());

		int res = rule.apply(program);
		assertEquals(1, res);

		assertFalse(pred.isNegated());

		Predicate pred2 = (Predicate) ((GringoRule) program.get(0)).getBody()
				.get(1); // b(X)
		assertFalse(pred2.isNegated());
		Predicate pred3 = (Predicate) ((GringoRule) program.get(0)).getBody()
				.get(2); // -c(X)
		assertTrue(pred3.isNegated());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2() throws Exception {
		String code = "a(X) :- b(X).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(2, literalList.size());
		assertEquals("a(X)", literalList.get(0).toString());
		assertEquals("b(X)", literalList.get(1).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(0, literalList.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnList() throws Exception {
		String code = "a(X,Y,Z) :- Z = X+Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
	}

	@Test
	public void testModifyLiteral_shouldSwitchNegated_1() throws Exception {
		String code = "a(X) :- X != 1, b(X), - c(X).";
		List<LineBase> program = parser.parse(code);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // -c(X)
		Predicate pred = (Predicate) lit;
		rule.setWriteComments(true);
		assertTrue(pred.isNegated());

		rule.modifyLiteral(lit);

		assertFalse(pred.isNegated());
	}

	@Test
	public void testModifyLiteral_shouldSwitchNegated_2() throws Exception {
		String code = "a(X) :- X != 1, b(X), c(X).";
		List<LineBase> program = parser.parse(code);
		Literal lit = ((GringoRule) program.get(0)).getBody().get(2); // c(X)
		Predicate pred = (Predicate) lit;
		rule.setWriteComments(true);
		assertFalse(pred.isNegated());

		rule.modifyLiteral(lit);

		assertTrue(pred.isNegated());

	}
}
