/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;

public class SwapTermModificationRuleTest {
	private GringoParser parser;
	private SwapTermModificationRule rule;

	@Before
	public void setUp() throws Exception {
		rule = new SwapTermModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testApply_shouldAddComment() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		assertNull(((GringoRule) program.get(0)).getHead().get(0)
				.getSucceedingComments());

		rule.setWriteComments(true);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals(1, ((GringoRule) program.get(0)).getHead().get(0)
				.getSucceedingComments().size());
	}

	@Test
	public void testApply_shouldDoNothing() throws Exception {
		String code = "a(X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);
		String oldString = program.get(0).toString();
		rule.setWriteComments(true);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());
		assertEquals(oldString, program.get(0).toString());
	}

	@Test
	public void testApply_shouldNotAddComment() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		assertNull(((GringoRule) program.get(0)).getHead().get(0)
				.getSucceedingComments());

		rule.setWriteComments(false);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(Y,X)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());

		assertNull(((GringoRule) program.get(0)).getHead().get(0)
				.getSucceedingComments());
	}

	@Test
	public void testApply_shouldNotSwap_1() throws Exception {
		String code = "a(X,X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(0, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(X,X)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldSwap_1() throws Exception {
		String code = "a(X,Y) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(Y,X)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldSwap_2() throws Exception {
		String code = "a(X) :- b(X,Y), a(Y), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("b(Y,X)", ((GringoRule) program.get(0)).getBody().get(0)
				.toString());
	}

	@Test
	public void testApply_shouldSwap_3() throws Exception {
		String code = "less(X,Y) :- n(X,Y),X < Y.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		rule.setRandom(mockedRandom);

		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertThat(res, is(1));
		assertThat(((GringoRule) program.get(0)).toString(),
				is("less(Y,X) :- n(X,Y), X < Y."));
	}

	@Test
	public void testApplyWithArityThree_shouldSwap_1() throws Exception {
		String code = "a(X,Y,Z) :- b(X), a(Y), a(Z), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(1));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(Y,X,Z)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApplyWithArityThree_shouldSwap_2() throws Exception {
		String code = "a(X,Y,Z) :- b(X), a(Y), a(Z), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(500000));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(X,Z,Y)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testApplyWithArityThree_shouldSwap_3() throws Exception {
		String code = "a(X,Y,Z) :- b(X), a(Y), a(Z), not c(X).";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setRandom(new Random(10000000));
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);

		assertEquals(1, res);

		assertNotNull(program);
		assertEquals(1, program.size());

		assertEquals("a(Z,Y,X)", ((GringoRule) program.get(0)).getHead().get(0)
				.toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "a(X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(0, result.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(X,X) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(0, result.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_3()
			throws Exception {
		String code = "a(X) :- b(X,X,X), not c(X).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(0, result.size());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnOneLiteral_1()
			throws Exception {
		String code = "a(X,Y) :- b(X), not c(X).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(1, result.size());
		assertEquals("a(X,Y)", result.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnOneLiteral_2()
			throws Exception {
		String code = "a(X) :- b(X,Y), not c(X).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(1, result.size());
		assertEquals("b(X,Y)", result.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnOneLiteral_3()
			throws Exception {
		String code = "a(X) :- b(X), b(Y), not c(X,Y).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(1, result.size());
		assertEquals("not c(X,Y)", result.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnOneLiteral_4()
			throws Exception {
		String code = "a(X,X) :- b(X), b(Y,Y,Y), not c(X,Y).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(1, result.size());
		assertEquals("not c(X,Y)", result.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturnTwoLiteral_1()
			throws Exception {
		String code = "a(X,Y) :- b(X,Y), not c(X).";
		List<LineBase> program = parser.parse(code);

		List<Literal> result = rule.findApplicableObjects(program);
		assertNotNull(result);
		assertEquals(2, result.size());
		assertEquals("a(X,Y)", result.get(0).toString());
		assertEquals("b(X,Y)", result.get(1).toString());
	}
}
