/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.modificationrules.gringo;

import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;
import static org.hamcrest.core.Is.is;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;
import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.modificationrules.TabuList;
import aspmodificator.parser.GringoParser;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.MathTerm;

public class TwiddleMathStatementModificationRuleTest {
	private GringoParser parser;
	private TwiddleMathStatementModificationRule rule;

	@Before
	public void setUp() throws Exception {
		rule = new TwiddleMathStatementModificationRule();
		parser = new GringoParser();
	}

	@Test
	public void testApply_shouldDoNothing() throws Exception {
		String code = "a(X) :- b(X), c(X).";
		List<LineBase> program = parser.parse(code);

		String original = program.get(0).toString();

		Random mockedRandom = mock(Random.class);
		rule.setRandom(mockedRandom);

		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);
		assertEquals(0, res);

		assertEquals(original, program.get(0).toString());
	}

	@Test
	public void testApply_shouldModify_1() throws Exception {
		String code = "a(X,Y,Z) :- Z = X + Y.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);

		// choose the second possible literal (Z = X ** Y)
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		// choose to modify the second term (X ** Y)
		when(mockedRandom.nextInt(2)).thenReturn(1);
		when(mockedRandom.nextBoolean()).thenReturn(true); // set to increment

		rule.setRandom(mockedRandom);
		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);

		int res = rule.apply(program);
		assertThat(res, is(1));

		assertThat(lit.toString(), is("Z = X+Y+1"));

		MathTerm mathTerm = (MathTerm) lit.getTerms().get(1);
		assertThat(mathTerm.toString(), is("X+Y+1"));
	}

	@Test
	public void testApply_shouldModify_2() throws Exception {
		String code = "a(X,Y,Z) :- Z = X ** Y.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		// choose the second possible literal (Z = X ** Y)
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		// choose to modify the second term (X ** Y)
		when(mockedRandom.nextInt(2)).thenReturn(1);
		when(mockedRandom.nextBoolean()).thenReturn(true); // set to increment

		rule.setRandom(mockedRandom);
		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		Literal lit = ((GringoRule) program.get(0)).getBody().get(0);

		int res = rule.apply(program);
		assertThat(res, is(1));

		assertThat(lit.toString(), is("Z = X**Y+1"));
	}

	@Test
	public void testApply_shouldModify_3() throws Exception {
		String code = ":- s(X,Y), X==(Y+X)*2.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		// choose the second possible literal (Z = X ** Y)
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		// choose to modify the second term (X ** Y)
		when(mockedRandom.nextInt(2)).thenReturn(0);
		when(mockedRandom.nextBoolean()).thenReturn(true); // set to increment

		rule.setRandom(mockedRandom);
		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);
		assertThat(res, is(1));
		assertThat(((GringoRule) program.get(0)).toString(),
				is(":- s(X,Y), X == (Y+1+X)*2."));
	}

	@Test
	public void testApply_shouldModify_4() throws Exception {
		String code = "serves(Rname,Dist) :- restaurant(Rname,RK), depot(Dname,DK), "
				+ "distance(RK,DK,Dist),D = #min [ distanceH1(DK1,RK,Y) = Y ], D == Dist.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		// choose the second possible literal (Z = X ** Y)
		when(mockedRandom.nextDouble()).thenReturn(0.0);
		// choose to modify the second term (X ** Y)
		when(mockedRandom.nextInt(2)).thenReturn(1);
		when(mockedRandom.nextBoolean()).thenReturn(true); // set to increment

		rule.setRandom(mockedRandom);
		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);
		assertThat(res, is(1));
		assertThat(((GringoRule) program.get(0)).getBody().get(3).toString(),
				is("D = #min [ distanceH1(DK1,RK,Y) = Y ]+1"));
	}

	@Test
	public void testApply_shouldModify_5() throws Exception {
		String code = "a(Y) :- b(X), Y = X +1.";
		List<LineBase> program = parser.parse(code);

		Random mockedRandom = mock(Random.class);
		// choose the second possible literal (Z = X ** Y)
		when(mockedRandom.nextDouble()).thenReturn(1.0).thenReturn(0.0);
		// choose to modify the second term (X ** Y)
		when(mockedRandom.nextInt(2)).thenReturn(0);
		when(mockedRandom.nextBoolean()).thenReturn(true); // set to increment

		rule.setRandom(mockedRandom);
		rule.setWriteComments(false);
		rule.setMinApplication(1);
		rule.setMaxApplication(1);

		int res = rule.apply(program);
		assertThat(res, is(1));
		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(Y) :- b(X), Y = X+1+1."));
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_1() throws Exception {
		String code = "a(X,Y) :- X = Y.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("X = Y", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn1_2() throws Exception {
		String code = "a(X,Y) :- a(X,Y-1).";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(1, literalList.size());
		assertEquals("Y-1", literalList.get(0).toString());
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_1() throws Exception {
		String code = "a(X,Y,Z) :- X = Y + Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertNotNull(literalList);
		assertEquals(2, literalList.size());
		assertThat(literalList.get(0).toString(), is("Y+Z"));
		assertThat(literalList.get(1).toString(), is("X = Y+Z"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_2() throws Exception {
		String code = "a(X,Y,Z) :- X = Y - Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(2));
		assertThat(literalList.get(0).toString(), is("Y-Z"));
		assertThat(literalList.get(1).toString(), is("X = Y-Z"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_3() throws Exception {
		String code = "a(X,Y,Z) :- X = Y ** Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(2));
		assertThat(literalList.get(0).toString(), is("Y**Z"));
		assertThat(literalList.get(1).toString(), is("X = Y**Z"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_4() throws Exception {
		String code = "a(X,Y,Z) :- X = Y * Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(2));
		assertThat(literalList.get(0).toString(), is("Y*Z"));
		assertThat(literalList.get(1).toString(), is("X = Y*Z"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturn2_5() throws Exception {
		String code = "a(X) :- X = 1 + 3.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(2));
		assertThat(literalList.get(0).toString(), is("1+3"));
		assertThat(literalList.get(1).toString(), is("X = 1+3"));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_1()
			throws Exception {
		String code = "";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(0));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_2()
			throws Exception {
		String code = "a(x). \n a(X) :- b(X). \n :- c(x).\n a(X) :- X == 1.";
		List<LineBase> program = parser.parse(code);

		rule.setIncludeRelLiterals(false);

		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(0));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_3()
			throws Exception {
		String code = "a(x). \n " + "a(X) :- b(X). \n " + ":- c(x).\n "
				+ "a(X) :- X = 1.";
		List<LineBase> program = parser.parse(code);

		rule.setIncludeRelLiterals(false);

		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(0));
	}

	@Test
	public void testFindApplicableObjects_shouldReturnEmptyList_4()
			throws Exception {
		String code = "a(x). \n " + "a(X) :- b(X). \n " + ":- c(x).\n "
				+ "a(X,Y) :- Y = 1 + X.";
		List<LineBase> program = parser.parse(code);

		rule.setIncludeMathTerms(false);
		rule.setIncludeRelLiterals(false);

		List<Literal> literalList = rule.findApplicableObjects(program);

		assertThat(literalList, is(notNullValue()));
		assertThat(literalList.size(), is(0));
	}

	@Test
	public void testModifyLiteral_shouldModify_1() throws Exception {
		String code = "a(X,Y,Z) :- X = Y / Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		rule.setWriteComments(false);
		rule.modifyLiteral(lit, 0, true);

		assertThat(lit.toString(), is("(Y+1)/Z"));
	}

	@Test
	public void testModifyLiteral_shouldModify_2() throws Exception {
		String code = "a(X,Y,Z) :- X = Y / Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		rule.setWriteComments(false);

		rule.modifyLiteral(lit, 1, true);

		assertThat(lit.toString(), is("Y/(Z+1)"));
	}

	@Test
	public void testModifyLiteral_shouldModify_3() throws Exception {
		String code = "a(X,Y,Z) :- X = Y / Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(1);

		rule.setWriteComments(false);

		rule.modifyLiteral(lit, 0, true);

		assertThat(lit.toString(), is("X+1 = Y/Z"));
	}

	@Test
	public void testModifyLiteral_shouldModify_4() throws Exception {
		String code = "a(X,Y,Z) :- X = Y / Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(1);

		rule.setWriteComments(false);

		rule.modifyLiteral(lit, 1, true);

		assertThat(lit.toString(), is("X = Y/Z+1"));
	}

	@Test
	public void testModifyLiteral_shouldModify_5() throws Exception {
		String code = "a(X,Y,Z) :- X = Y / Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(1);

		rule.setWriteComments(false);

		rule.modifyLiteral(lit, 1, false);

		assertThat(lit.toString(), is("X = Y/Z-1"));
	}

	@Test
	public void testModifyLiteral_shouldModify_6() throws Exception {
		String code = "a(X,Y,Z) :- X = Y / Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		rule.setWriteComments(false);

		rule.modifyLiteral(lit, 0, false);

		assertThat(lit.toString(), is("(Y-1)/Z"));
	}

	@Test
	public void testModifyLiteral_shouldModify_7() throws Exception {
		String code = "a(X,Y,Z) :- X = Y - Z.";
		List<LineBase> program = parser.parse(code);
		List<Literal> literalList = rule.findApplicableObjects(program);
		Literal lit = literalList.get(0);

		rule.setWriteComments(false);

		rule.modifyLiteral(lit, 0, false);

		assertThat(lit.toString(), is("Y-1-Z"));
	}

	@Test
	public void testTabuList_shouldCreateDifferentMutants_1() throws Exception {
		String code = "a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y * Z.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMaxApplication(1);
		rule.setMinApplication(1);
		rule.setTabuList(new TabuList());
		rule.setUseMemory(true);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextInt(2)).thenReturn(0).thenReturn(0).thenReturn(1);
		when(mockedRandom.nextDouble()).thenReturn(0.0).thenReturn(0.0);

		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		// generate first mutant
		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = (Y+1)*Z."));

		// generate second mutant
		program = parser.parse(code);
		res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y*(Z+1)."));
	}

	@Test
	public void testTabuList_shouldCreateDifferentMutants_2() throws Exception {
		String code = "a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y * Z.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMaxApplication(1);
		rule.setMinApplication(1);
		rule.setTabuList(new TabuList());
		rule.setUseMemory(true);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextInt(2)).thenReturn(0).thenReturn(0).thenReturn(0)
				.thenReturn(0);
		when(mockedRandom.nextDouble()).thenReturn(0.0).thenReturn(0.0);

		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true)
				.thenReturn(true);
		rule.setRandom(mockedRandom);

		// generate first mutant
		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = (Y+1)*Z."));

		// generate second mutant
		program = parser.parse(code);
		res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y*Z+1."));
	}

	@Test
	public void testTabuList_shouldCreateDifferentMutants_3() throws Exception {
		String code = "a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y * Z.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMaxApplication(1);
		rule.setMinApplication(1);
		rule.setTabuList(new TabuList());
		rule.setUseMemory(true);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextInt(2)).thenReturn(0).thenReturn(0).thenReturn(0)
				.thenReturn(0).thenReturn(0).thenReturn(0).thenReturn(1);
		when(mockedRandom.nextDouble()).thenReturn(0.0).thenReturn(0.0);

		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true)
				.thenReturn(true);
		rule.setRandom(mockedRandom);

		// generate first mutant
		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = (Y+1)*Z."));

		// generate second mutant
		program = parser.parse(code);
		res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y*Z+1."));
	}

	@Test
	public void testTabuList_shouldCreateEqualMutants_1() throws Exception {
		String code = "a(X,Y,Z) :- b(X), b(Y), b(Z), X = Y * Z.";
		List<LineBase> program = parser.parse(code);

		rule.setWriteComments(false);
		rule.setMaxApplication(1);
		rule.setMinApplication(1);
		rule.setTabuList(new TabuList());
		rule.setUseMemory(false);

		Random mockedRandom = mock(Random.class);
		when(mockedRandom.nextInt(2)).thenReturn(0).thenReturn(0).thenReturn(1);
		when(mockedRandom.nextDouble()).thenReturn(0.0).thenReturn(0.0);

		when(mockedRandom.nextBoolean()).thenReturn(true).thenReturn(true);
		rule.setRandom(mockedRandom);

		// generate first mutant
		int res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = (Y+1)*Z."));

		// generate second mutant
		program = parser.parse(code);
		res = rule.apply(program);

		assertThat(res, is(1));

		assertThat(((GringoRule) program.get(0)).toString(),
				is("a(X,Y,Z) :- b(X), b(Y), b(Z), X = (Y+1)*Z."));
	}

}
