/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package aspmodificator.parser;

import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsNull.notNullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;
import static org.hamcrest.core.IsNull.*;

import java.util.List;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.model.LineBase;
import aspmodificator.parser.gringo.AggregateLiteral;
import aspmodificator.parser.gringo.ArgTerm;
import aspmodificator.parser.gringo.ComputeCommand;
import aspmodificator.parser.gringo.ComputeLiteral;
import aspmodificator.parser.gringo.ConstDefinition;
import aspmodificator.parser.gringo.GringoRule;
import aspmodificator.parser.gringo.Literal;
import aspmodificator.parser.gringo.MetaStatement;
import aspmodificator.parser.gringo.Optimize;
import aspmodificator.parser.gringo.OptimizeCommand;
import aspmodificator.parser.gringo.OptimizeLiteral;
import aspmodificator.parser.gringo.PoolTerm;
import aspmodificator.parser.gringo.Predicate;
import aspmodificator.parser.gringo.RangeTerm;
import aspmodificator.parser.gringo.RelLit;

public class GringoParserTest {
	private GringoParser parser;

	@Test
	public void bugFix_1() throws Exception {
		String text = "sol((P*M)+R,(Q*M)+S,Z) :- int(P), int(M), int(R), int(Q), int(M), int(S), int(Z).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertEquals(1, result.size());
		Literal sol = ((GringoRule) result.get(0)).getHead().get(0);
		assertNotNull(sol);
		assertEquals("sol((P*M)+R,(Q*M)+S,Z)", sol.toString());

	}

	@Test
	public void parseAggregate_1() throws Exception {
		String text = "3 { enroll(C) : course(C,S,H) } 6.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		AggregateLiteral aggLit = (AggregateLiteral) rule.getHead().get(0);
		assertNotNull(aggLit);

		assertEquals("3", aggLit.getLower().toString());
		assertEquals("6", aggLit.getUpper().toString());
		assertNotNull(aggLit.getTerms());
		assertEquals(1, aggLit.getTerms().size());
		assertEquals("enroll(C) : course(C,S,H)", aggLit.getTerms().get(0)
				.toString());

		List<Literal> body = rule.getBody();
		assertNull(body);
	}

	@Test
	public void parseAggregate_10() throws Exception {
		String text = "c(Result) :- Result = #sum [a(X)=1, not b(X)=1].";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);

		List<Literal> head = rule.getHead();
		assertNotNull(head);

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		RelLit relLit = (RelLit) body.get(0);
		assertNotNull(relLit);

		AggregateLiteral aggLit = (AggregateLiteral) relLit.getTerms().get(1);
		assertNotNull(aggLit);

		assertNull(aggLit.getLower());
		assertNull(aggLit.getUpper());
		assertNotNull(aggLit.getTerms());
		assertEquals(2, aggLit.getTerms().size());
		assertEquals("a(X) = 1", aggLit.getTerms().get(0).toString());
		assertEquals("not b(X) = 1", aggLit.getTerms().get(1).toString());
		assertEquals("#sum", aggLit.getIdentifier());
	}

	@Test
	public void parseAggregate_11() throws Exception {
		String code = "	{ push(B,right,B1,S) } :- reachable(L,S), right(L,B), box_step(B,S), pushable_right(B,B1,S), good_pushlocation(B1), actionstep(S).";

		List<LineBase> result = parser.parse(code);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));
		GringoRule rule = (GringoRule) result.get(0);

		List<Literal> head = rule.getHead();
		assertNotNull(head);
		AggregateLiteral aggLit = (AggregateLiteral) head.get(0);
		assertNotNull(aggLit);

		assertNotNull(aggLit.getTerms());
		assertEquals(1, aggLit.getTerms().size());
		assertEquals("push(B,right,B1,S)", aggLit.getTerms().get(0).toString());

	}

	@Test
	public void parseAggregate_2() throws Exception {
		String text = "{ enroll(C) : course(C,S,H) } 6.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		AggregateLiteral aggLit = (AggregateLiteral) rule.getHead().get(0);
		assertNotNull(aggLit);

		assertNull(aggLit.getLower());
		assertEquals("6", aggLit.getUpper().toString());
		assertNotNull(aggLit.getTerms());
		assertEquals(1, aggLit.getTerms().size());
		assertEquals("enroll(C) : course(C,S,H)", aggLit.getTerms().get(0)
				.toString());

		List<Literal> body = rule.getBody();
		assertNull(body);
	}

	@Test
	public void parseAggregate_3() throws Exception {
		String text = "a(X) :- X = #min [ enroll(C) : hours(C,H) = H ].";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		RelLit relLit = (RelLit) body.get(0);
		assertNotNull(relLit);

		AggregateLiteral aggLit = (AggregateLiteral) relLit.getTerms().get(1);
		assertNotNull(aggLit);

		assertNull(aggLit.getLower());
		assertNull(aggLit.getUpper());
		assertNotNull(aggLit.getTerms());
		assertEquals(1, aggLit.getTerms().size());
		assertEquals("enroll(C) : hours(C,H) = H", aggLit.getTerms().get(0)
				.toString());

		assertEquals("#min", aggLit.getIdentifier());
	}

	@Test
	public void parseAggregate_4() throws Exception {
		String text = ":- 1 #sum [a(X)=1, not b(X)=1] 1.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		AggregateLiteral aggLit = (AggregateLiteral) body.get(0);
		assertNotNull(aggLit);

		assertEquals("1", aggLit.getLower().toString());
		assertEquals("1", aggLit.getUpper().toString());
		assertNotNull(aggLit.getTerms());
		assertEquals(2, aggLit.getTerms().size());
		assertEquals("a(X) = 1", aggLit.getTerms().get(0).toString());
		assertEquals("not b(X) = 1", aggLit.getTerms().get(1).toString());

		assertEquals("#sum", aggLit.getIdentifier());
	}

	@Test
	public void parseAggregate_5() throws Exception {
		String text = "c(Result) :- Result = #count {a(X), not b(X)}.";

		List<LineBase> result = parser.parse(text);
		assertEquals(1, result.size());
		GringoRule rule = (GringoRule) result.get(0);

		List<Literal> head = rule.getHead();
		assertNotNull(head);

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		RelLit relLit = (RelLit) body.get(0);
		assertNotNull(relLit);

		AggregateLiteral aggLit = (AggregateLiteral) relLit.getTerms().get(1);
		assertNotNull(aggLit);

		assertNull(aggLit.getLower());
		assertNull(aggLit.getUpper());
		assertNotNull(aggLit.getTerms());
		assertEquals(2, aggLit.getTerms().size());
		assertEquals("a(X)", aggLit.getTerms().get(0).toString());
		assertEquals("not b(X)", aggLit.getTerms().get(1).toString());
		assertEquals("#count", aggLit.getIdentifier());
	}

	@Test
	public void parseAggregate_6() throws Exception {
		String text = ":- 1 [a(X)=2, not b(X)=1] 1.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		AggregateLiteral aggLit = (AggregateLiteral) body.get(0);
		assertNotNull(aggLit);

		assertEquals("1", aggLit.getLower().toString());
		assertEquals("1", aggLit.getUpper().toString());
		assertNotNull(aggLit.getTerms());
		assertEquals(2, aggLit.getTerms().size());
		assertEquals("a(X) = 2", aggLit.getTerms().get(0).toString());
		assertEquals("not b(X) = 1", aggLit.getTerms().get(1).toString());

		assertNull(aggLit.getIdentifier());
	}

	@Test
	public void parseAggregate_7() throws Exception {
		String code = ":- 1 [ a(C) : b(C) ] 1.";
		List<LineBase> result = parser.parse(code);

		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		AggregateLiteral aggLit = (AggregateLiteral) body.get(0);
		assertNotNull(aggLit);

		assertEquals("1", aggLit.getLower().toString());
		assertEquals("1", aggLit.getUpper().toString());
		assertNotNull(aggLit.getTerms());
		assertEquals(1, aggLit.getTerms().size());
		assertEquals("a(C) : b(C)", aggLit.getTerms().get(0).toString());

		assertNull(aggLit.getIdentifier());
	}

	@Test
	public void parseAggregate_8() throws Exception {
		String code = "1 { push(X,e;w,T) : X=1..M, push(Y,n;s,T) : Y=1..N } 1 :- num_rows(M), num_cols(N), max_steps(S), T = 1..S, neg_goal(T-1).";
		List<LineBase> result = parser.parse(code);

		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);

		List<Literal> head = rule.getHead();
		assertNotNull(head);
		AggregateLiteral aggLit = (AggregateLiteral) head.get(0);
		assertNotNull(aggLit);

		assertEquals("1", aggLit.getLower().toString());
		assertEquals("1", aggLit.getUpper().toString());
		assertNotNull(aggLit.getTerms());
		assertEquals(2, aggLit.getTerms().size());
		assertEquals("push(X,e;w,T) : X = 1..M", aggLit.getTerms().get(0)
				.toString());
		assertEquals("push(Y,n;s,T) : Y = 1..N", aggLit.getTerms().get(1)
				.toString());
	}

	@Test
	public void parseAggregate_9() throws Exception {
		String code = ":- W+1 [ cycle(X,Y) : cost(X,Y,C) = C ], maxweight(W).";
		List<LineBase> result = parser.parse(code);

		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);

		List<Literal> head = rule.getHead();
		assertNull(head);
		List<Literal> body = rule.getBody();

		AggregateLiteral aggLit = (AggregateLiteral) body.get(0);
		assertNotNull(aggLit);

		assertNotNull(aggLit.getLower());
		assertEquals("W+1", aggLit.getLower().toString());
		assertNull(aggLit.getUpper());
		assertNotNull(aggLit.getTerms());
		assertEquals(1, aggLit.getTerms().size());
		assertEquals("cycle(X,Y) : cost(X,Y,C) = C", aggLit.getTerms().get(0)
				.toString());
	}

	@Test
	public void parseAssignment_1() throws Exception {
		String text = "a(X) :- X=1.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);

		RelLit term = (RelLit) body.get(0);
		assertNotNull(term);
	}

	@Test
	public void parseComment_01() throws Exception {
		String text = "% take square root";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
	}

	@Test
	public void parseComment_02() throws Exception {
		String text = "%* take square root *%";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
	}

	@Test
	public void parseComment_03() throws Exception {
		String text = "a(X,Y) :-  a(%* blablabla *%Y,X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
	}

	@Test
	public void parseComment_04() throws Exception {
		String text = "a(X,Y) :- a(Y,X).% blablabla\n";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 2);
	}

	@Test
	public void parseComment_05() throws Exception {
		String text = "a(X,Y) :- a(Y,X).% blablabla\n"
				+ "b(X,Y,Z):- a(X), a(Y), a(Z), c(123).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 3);
	}

	@Test
	public void parseComment_05_2() throws Exception {
		String text = "b(X,Y,Z):- a(X), a(Y), a(Z), c(cons).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
	}

	@Test
	public void parseComment_06() throws Exception {
		String text = "a(X,Y) :- a(Y,X).% bla\n%bla\n%bla\n"
				+ "b(X,Y,Z):- a(X), a(Y), a(Z), c(con).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 5);
	}

	@Test
	public void parseComment_07() throws Exception {
		String text = "% take square root\n" + "subSize(M) :- size(M).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 2);
	}

	@Test
	public void parseComment_08() throws Exception {
		String text = "%* blablabla *%";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
	}

	@Test
	public void parseComment_09() throws Exception {
		String text = "a(X,Y) :- a(Y,X). %* blablabla *%";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 2);
	}

	@Test
	public void parseComment_10() throws Exception {
		String text = "%* blablabla *%\na(X,Y) :- a(Y,X). ";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 2);
	}

	@Test
	public void parseComment_11() throws Exception {
		String text = "a(X,Y) :- %* blablabla *% a(Y,X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule r = (GringoRule) result.get(0);
		Literal bodyLiteral = r.getBody().get(0);

		assertNotNull(bodyLiteral.getPrecedingComments());
		assertEquals(1, bodyLiteral.getPrecedingComments().size());
		assertEquals("%* blablabla *%",
				bodyLiteral.getPrecedingComments().get(0).getText());
	}

	@Test
	public void parseComment_12() throws Exception {
		String text = "a(X,Y) :- %* 11111  %* 2222 *%*%  a(Y,X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule r = (GringoRule) result.get(0);
		Literal bodyLiteral = r.getBody().get(0);

		assertNotNull(bodyLiteral.getPrecedingComments());
		assertEquals(1, bodyLiteral.getPrecedingComments().size());
		assertEquals("%* 11111  %* 2222 *%*%", bodyLiteral
				.getPrecedingComments().get(0).getText());

	}

	@Test
	public void parseComment_13() throws Exception {
		String text = "a(X,Y) :- %* 11111  *% %* 2222 *%  a(Y,X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule r = (GringoRule) result.get(0);
		Literal bodyLiteral = r.getBody().get(0);

		assertNotNull(bodyLiteral.getPrecedingComments());
		assertEquals(2, bodyLiteral.getPrecedingComments().size());
		assertEquals("%* 11111  *%", bodyLiteral.getPrecedingComments().get(0)
				.getText());
		assertEquals("%* 2222 *%", bodyLiteral.getPrecedingComments().get(1)
				.getText());

	}

	@Test
	public void parseCompleteProgram_1() throws Exception {
		String text = "% take square root\n"
				+ "subSize(M) :- size(N), M = 0..N, M*M == N.\n"
				+ "\n"
				+ "% each cell contains exactly one number\n"
				+ "1 { sol(X,Y,Z) : Z = 1..N } 1 :- size(N), X = 1..N, Y = 1..N.\n"
				+ "\n"
				+ "% each row contains each number exactly once\n"
				+ "1 { sol(X,Y,Z) : Y = 1..N } 1 :- size(N), X = 1..N, Z = 1..N.\n"
				+ "\n"
				+ "% each column contains each number exactly once\n"
				+ "1 { sol(X,Y,Z) : X = 1..N } 1 :- size(N), Y = 1..N, Z = 1..N.\n"
				+ "\n"
				+ "% each subsquare contains each number exactly once\n"
				+ "1 { sol((P*M)+R,(Q*M)+S,Z) : R = 1..M : S = 1..M } 1 :- subSize(M), P = 0..M-1, Q = 0..M-1, size(N), Z = 1..N.\n"
				+ "\n" + ":- square(X,Y,Z), not sol(X,Y,Z).\n" + "\n"
				+ "#hide.\n" + "#show sol(X,Y,Z).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertEquals(13, result.size());

	}

	@Test
	public void parseComputeCommand_1() throws Exception {
		String text = "#compute 0 { p(X) : q(X) }.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		ComputeCommand cc = (ComputeCommand) result.get(0);

		assertThat(cc.getComputeLiteralList(), is(notNullValue()));
		assertThat(cc.getComputeLiteralList().size(), is(1));
		assertThat(cc.getNumber(), is(notNullValue()));
		assertThat(cc.toString(), is(text));
	}

	@Test
	public void parseComputeCommand_2() throws Exception {
		String text = "#compute { p(X) : q(X) }.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		ComputeCommand cc = (ComputeCommand) result.get(0);

		assertThat(cc.getComputeLiteralList(), is(notNullValue()));
		assertThat(cc.getComputeLiteralList().size(), is(1));
		assertThat(cc.getNumber(), is(nullValue()));
		assertThat(cc.toString(), is(text));
	}

	@Test
	public void parseComputeCommand_3() throws Exception {
		String text = "compute { p(X) : q(X) : y(X) }.";
		String text2 = "#compute { p(X) : q(X) : y(X) }.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		ComputeCommand cc = (ComputeCommand) result.get(0);

		assertThat(cc.getComputeLiteralList(), is(notNullValue()));
		assertThat(cc.getComputeLiteralList().size(), is(1));
		assertThat(cc.getNumber(), is(nullValue()));
		assertThat(cc.toString(), is(text2));
	}

	@Test
	public void parseComputeCommand_4() throws Exception {
		String text = "compute 2 { p(X) : q(X), pp(Y) : qq(Y) }.";
		String text2 = "#compute 2 { p(X) : q(X), pp(Y) : qq(Y) }.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		ComputeCommand cc = (ComputeCommand) result.get(0);

		assertThat(cc.getComputeLiteralList(), is(notNullValue()));
		assertThat(cc.getComputeLiteralList().size(), is(2));
		assertThat(cc.getNumber(), is(notNullValue()));
		assertThat(cc.getNumber(), is(2));
		assertThat(cc.toString(), is(text2));
	}

	@Test
	public void parseComputeLiteral_1() throws Exception {
		String text = "#compute 0 { p(X) : q(X) }.";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		ComputeCommand cc = (ComputeCommand) result.get(0);
		assertThat(cc.getComputeLiteralList(), is(notNullValue()));
		assertThat(cc.getComputeLiteralList().size(), is(1));
		assertThat(cc.getNumber(), is(notNullValue()));

		ComputeLiteral cLit = (ComputeLiteral) cc.getComputeLiteralList()
				.get(0);
		assertThat(cLit, is(notNullValue()));

		assertThat(cLit.toString(), is("p(X) : q(X)"));
	}

	@Test
	public void parseConstant_1() throws Exception {
		String text = "#const x = 42.";

		List<LineBase> result = parser.parse(text);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));

		ConstDefinition cd = (ConstDefinition) result.get(0);
		assertThat(cd.toOutputString(), is(text));
		assertThat(cd.toString(), is(text));
	}

	@Test
	public void parseConstant_2() throws Exception {
		String text = "const y = f(g, h).";
		String text2 = "#const y = f(g, h).";

		List<LineBase> result = parser.parse(text);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));

		ConstDefinition cd = (ConstDefinition) result.get(0);
		assertThat(cd.toOutputString(), is(text2));
		assertThat(cd.toString(), is(text2));
	}

	@Test
	public void parseConstraint() throws Exception {
		String text = " :- move(T,D,X,Y), stone(X,Y,T), dir(D,DX,DY).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(3, body.size());
	}

	@Test
	public void parseDisjunction() throws Exception {
		String text = "flies(X) | -flies(X) :- bird(X).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(1, body.size());
	}

	@Test
	public void parseFact() throws Exception {
		String text = "dir(left, 1, 0).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		assertNull(rule.getBody());
	}

	@Test
	public void parseFact2() throws Exception {
		String text = "c(1234).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		assertNull(rule.getBody());
	}

	@Test
	public void parseGroundedRule() throws Exception {
		String text = "a(x) :- b(x).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(1, body.size());
	}

	@Test
	public void parseGroundedRule2() throws Exception {
		String text = "a(X) :- b(X), c(constant).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(2, body.size());
	}

	@Test
	public void parseGroundedRule3() throws Exception {
		String text = "a :- b, not c.";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(2, body.size());
	}

	@Test
	public void parseHide_1() throws Exception {
		String text = "#hide p/3.";

		List<LineBase> result = parser.parse(text);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));

		MetaStatement ms = (MetaStatement) result.get(0);
		assertThat(ms.toOutputString(), is(text));
		assertThat(ms.toString(), is(text));
	}

	@Test
	public void parseHide_2() throws Exception {
		String text = "hide p(X,Y,Z).";
		String text2 = "#hide p(X,Y,Z).";

		List<LineBase> result = parser.parse(text);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));

		MetaStatement ms = (MetaStatement) result.get(0);
		assertThat(ms.toOutputString(), is(text2));
		assertThat(ms.toString(), is(text2));
	}

	@Test
	public void parseMultiplication() throws Exception {
		String text = "result(A,B,C) :- C = A * B.";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);

		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(1, body.size());
	}

	@Test
	public void parseNormalRule() throws Exception {
		String text = "a(X) :- b(X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(1, body.size());
	}

	@Test
	public void parseOptimization_1() throws Exception {
		String text = "maximize [ hotel(X) : star(X,Y) = Y ].";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		OptimizeCommand rule = (OptimizeCommand) result.get(0);

		assertNotNull(rule.getType());
		assertEquals(Optimize.MAXIMIZE, rule.getType());

		List<OptimizeLiteral> literals = rule.getLiterals();
		assertNotNull(literals);
		assertEquals(1, literals.size());
	}

	@Test
	public void parseOptimization_2() throws Exception {
		String text = "minimize [ hotel(X) : cost(X,Y) : star(X,Z) = Y/Z ].";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		OptimizeCommand rule = (OptimizeCommand) result.get(0);
		assertNotNull(rule.getType());
		assertEquals(Optimize.MINIMIZE, rule.getType());

		List<OptimizeLiteral> literals = rule.getLiterals();
		assertNotNull(literals);
		assertEquals(1, literals.size());
	}

	@Test
	public void parsePooling_1() throws Exception {
		String text = "p(a;b).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		Predicate pred = (Predicate) rule.getHead().get(0);
		assertThat(pred, notNullValue());

		assertThat(pred.getIdentifier(), is("p"));

		PoolTerm poolTerm = (PoolTerm) pred.getTerms().get(0);
		assertThat(poolTerm, notNullValue());

		assertThat(poolTerm.getIdentifier(), is(";"));
		assertThat(poolTerm.getArity(), is(2));
		assertThat(poolTerm.getTerms().get(0).getIdentifier(), is("a"));
		assertThat(poolTerm.getTerms().get(1).getIdentifier(), is("b"));
	}

	@Test
	public void parsePooling_2() throws Exception {
		String text = "p(x,a;b).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		Predicate pred = (Predicate) rule.getHead().get(0);
		assertThat(pred, notNullValue());

		assertThat(pred.getIdentifier(), is("p"));

		PoolTerm poolTerm = (PoolTerm) pred.getTerms().get(1);
		assertThat(poolTerm, notNullValue());

		assertThat(poolTerm.getIdentifier(), is(";"));
		assertThat(poolTerm.getArity(), is(2));
		assertThat(poolTerm.getTerms().get(0).getIdentifier(), is("a"));
		assertThat(poolTerm.getTerms().get(1).getIdentifier(), is("b"));
	}

	@Test
	public void parsePooling_3() throws Exception {
		String text = "p(x,a;b,y).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		Predicate pred = (Predicate) rule.getHead().get(0);
		assertThat(pred, notNullValue());

		assertThat(pred.getIdentifier(), is("p"));

		PoolTerm poolTerm = (PoolTerm) pred.getTerms().get(1);
		assertThat(poolTerm, notNullValue());

		assertThat(poolTerm.getIdentifier(), is(";"));
		assertThat(poolTerm.getArity(), is(2));
		assertThat(poolTerm.getTerms().get(0).getIdentifier(), is("a"));
		assertThat(poolTerm.getTerms().get(1).getIdentifier(), is("b"));
	}

	@Test
	public void parsePooling_4() throws Exception {
		String text = "p(a,b;;x,y).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		Predicate pred = (Predicate) rule.getHead().get(0);
		assertThat(pred, notNullValue());

		assertThat(pred.getIdentifier(), is("p"));

		ArgTerm argTerm = (ArgTerm) pred.getTerms().get(1);
		assertThat(argTerm, notNullValue());

		assertThat(argTerm.getIdentifier(), is(";;"));
		assertThat(argTerm.getArity(), is(2));
		assertThat(argTerm.getTerms().get(0).getIdentifier(), is("x"));
		assertThat(argTerm.getTerms().get(1).getIdentifier(), is("y"));
	}

	@Test
	public void parsePooling_5() throws Exception {
		String text = "p(a;b;c).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		Predicate pred = (Predicate) rule.getHead().get(0);
		assertThat(pred, notNullValue());

		assertThat(pred.getIdentifier(), is("p"));

		PoolTerm poolTerm = (PoolTerm) pred.getTerms().get(0);
		assertThat(poolTerm, notNullValue());

		assertThat(poolTerm.getIdentifier(), is(";"));
		assertThat(poolTerm.getArity(), is(2));
		assertThat(poolTerm.getTerms().get(0).getIdentifier(), is(";"));
		assertThat(poolTerm.getTerms().get(1).getIdentifier(), is("c"));
	}

	@Test
	public void parseRange_1() throws Exception {
		String text = "a(1..5).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		assertNull(rule.getBody());

		Literal l = rule.getHead().get(0);
		assertEquals("a", l.getIdentifier());
		Literal literal = l.getTerms().get(0);
		assertEquals(RangeTerm.class, literal.getClass());
		assertEquals("1..5", literal.toString());
	}

	@Test
	public void parseRange_2() throws Exception {
		String text = "a(-1..5).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		assertNull(rule.getBody());

		Literal l = rule.getHead().get(0);
		assertEquals("a", l.getIdentifier());
		Literal literal = l.getTerms().get(0);
		assertEquals(RangeTerm.class, literal.getClass());
		assertEquals("-1..5", literal.toString());
	}

	@Test
	public void parseRange_3() throws Exception {
		String text = "field(-3..X-1) :- num(X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		assertNotNull(rule.getBody());

		Literal l = rule.getHead().get(0);
		assertEquals("field", l.getIdentifier());
		Literal literal = l.getTerms().get(0);
		assertEquals(RangeTerm.class, literal.getClass());
		assertEquals("-3..X-1", literal.toString());
	}

	@Test
	public void parseRange_4() throws Exception {
		String text = "field(-3..X-1-1) :- num(X).";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		assertNotNull(rule.getBody());

		Literal l = rule.getHead().get(0);
		assertEquals("field", l.getIdentifier());
		Literal literal = l.getTerms().get(0);
		assertEquals(RangeTerm.class, literal.getClass());
		assertEquals("-3..X-1-1", literal.toString());
	}

	@Test
	public void parseShow_1() throws Exception {
		String text = "#show p/3.";

		List<LineBase> result = parser.parse(text);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));

		MetaStatement ms = (MetaStatement) result.get(0);
		assertThat(ms.toOutputString(), is(text));
		assertThat(ms.toString(), is(text));
	}

	@Test
	public void parseShow_2() throws Exception {
		String text = "show p(X,Y,Z).";
		String text2 = "#show p(X,Y,Z).";

		List<LineBase> result = parser.parse(text);

		assertThat(result, is(notNullValue()));
		assertThat(result.size(), is(1));

		MetaStatement ms = (MetaStatement) result.get(0);

		assertThat(ms.toOutputString(), is(text2));
		assertThat(ms.toString(), is(text2));
	}

	@Test
	public void parseStrongNegation() throws Exception {
		String text = " -move(X,Y) :- nofuell(X).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(1, body.size());
	}

	@Test
	public void parseStrongNegation_2() throws Exception {
		String text = " move(X,Y) :- nofuell(X), not -a(X).";

		List<LineBase> result = parser.parse(text);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		assertNotNull(rule.getHead());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(2, body.size());
	}

	@Test
	public void parseSubtraction() throws Exception {
		String text = "result(A,B,C) :- C = A - B.";

		List<LineBase> result = parser.parse(text);
		assertNotNull(result);
		assertTrue(result.size() == 1);
		GringoRule rule = (GringoRule) result.get(0);
		List<Literal> head = rule.getHead();
		assertNotNull(head);
		assertEquals(1, head.size());

		List<Literal> body = rule.getBody();
		assertNotNull(body);
		assertEquals(1, body.size());
	}

	@Before
	public void setUp() {
		parser = new GringoParser();
	}
}
