/*******************************************************************************
 * Copyright (c) 2011 Martin Schwengerer.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Martin Schwengerer - initial API and implementation
 ******************************************************************************/
package main;

import static org.junit.Assert.*;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import aspmodificator.modificationrules.IModificationRule;
import aspmodificator.modificationrules.gringo.AbstractModificationRule;
import aspmodificator.modificationrules.gringo.AggregateLiteralModificationRule;
import aspmodificator.modificationrules.gringo.ConstantToVariableModificationRule;
import aspmodificator.modificationrules.gringo.DeleteRuleModificationRule;
import aspmodificator.modificationrules.gringo.DeleteTermModificationRule;
import aspmodificator.modificationrules.gringo.MathOperatorsModificationRule;
import aspmodificator.modificationrules.gringo.RangeTermModificationRule;
import aspmodificator.modificationrules.gringo.RelationOperatorModificationRule;
import aspmodificator.modificationrules.gringo.RenamePredicateModificationRule;
import aspmodificator.modificationrules.gringo.ReplaceConstantModificationRule;
import aspmodificator.modificationrules.gringo.ReplaceLiteralModificationRule;
import aspmodificator.modificationrules.gringo.SwapStrongNegationModificationRule;
import aspmodificator.modificationrules.gringo.SwapTermModificationRule;
import aspmodificator.modificationrules.gringo.SwitchDefaultNegationModificationRule;
import aspmodificator.modificationrules.gringo.VariableToConstantModificationRule;
import static org.hamcrest.CoreMatchers.*;
import static org.hamcrest.core.Is.is;

public class AspModificationConfigTest {

	private File createTestFile(String string) throws IOException {
		File tmpFile = File.createTempFile("test", null);

		BufferedWriter out = new BufferedWriter(new FileWriter(tmpFile));
		out.write(string);
		out.close();
		return tmpFile;
	}

	@Before
	public void setUp() throws Exception {
	}

	@Test
	public void testCreateConfigFromFile_aggregatLiteralMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeAggregates\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(AggregateLiteralModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_AvoidNearlyInequalOperators_shouldReturnFalse_1()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((RelationOperatorModificationRule) modRules.get(0))
						.isAvoidNearlyInequalOperators(),
				is(false));
	}

	@Test
	public void testCreateConfigFromFile_AvoidNearlyInequalOperators_shouldReturnFalse_2()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"1\" avoidNearlyInequalOperators=\"false\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((RelationOperatorModificationRule) modRules.get(0))
						.isAvoidNearlyInequalOperators(),
				is(false));
	}

	@Test
	public void testCreateConfigFromFile_AvoidNearlyInequalOperators_shouldReturnTrue()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"1\" avoidNearlyInequalOperators=\"true\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((RelationOperatorModificationRule) modRules.get(0))
						.isAvoidNearlyInequalOperators(),
				is(true));
	}

	@Test
	public void testCreateConfigFromFile_changeConstantToVariableMR()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeConstantToVariable\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0),
				is(ConstantToVariableModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_changeVariableToConstantMR()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeVariableToConstant\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0),
				is(VariableToConstantModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_deleteLiteralRule() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule1\" type=\"deleteLiteral\" min=\"2\" max=\"2\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(DeleteTermModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_deleteRuleRule() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(DeleteRuleModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_mathOperatorsMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeMathOperators\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(MathOperatorsModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_max_shouldReturn1() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((AbstractModificationRule) modRules.get(0))
						.getMaxApplication(),
				is(1));
	}

	@Test
	public void testCreateConfigFromFile_max_shouldReturn2() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"2\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((AbstractModificationRule) modRules.get(0))
						.getMaxApplication(),
				is(2));
	}

	@Test
	public void testCreateConfigFromFile_min_shouldReturn1() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((AbstractModificationRule) modRules.get(0))
						.getMinApplication(),
				is(1));
	}

	@Test
	public void testCreateConfigFromFile_min_shouldReturn3() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"3\" max=\"5\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
		assertThat(
				((AbstractModificationRule) modRules.get(0))
						.getMinApplication(),
				is(3));
	}

	@Test
	public void testCreateConfigFromFile_noCopies_shouldReturn10()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <noCopies value=\"10\"/>   \n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.getNoCopies(), is(10));
	}

	@Test
	public void testCreateConfigFromFile_noCopies_shouldReturn100()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <noCopies value= \"100\" />   \n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.getNoCopies(), is(100));
	}

	@Test
	public void testCreateConfigFromFile_noCopies_shouldReturnDefaultValue()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.getNoCopies(), is(1));
	}

	@Test
	public void testCreateConfigFromFile_rangeTermMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRangeTerms\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RangeTermModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_relationOperatorMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"changeRelationOperators\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RelationOperatorModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_renamePredicatesMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"renamePredicates\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(RenamePredicateModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_replaceConstantMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"replaceConstants\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(ReplaceConstantModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_replaceLiteralMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"replaceGroundedLiterals\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(ReplaceLiteralModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_ruleApplication_shouldReturnAll_1()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <ruleApplication application=\"all\"/>   \n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.getRuleApplication(), is(RuleApplicationEnum.all));
	}

	@Test
	public void testCreateConfigFromFile_ruleApplication_shouldReturnAll_2()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.getRuleApplication(), is(RuleApplicationEnum.all));
	}

	@Test
	public void testCreateConfigFromFile_ruleApplication_shouldReturnOneOfMany()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <ruleApplication application=\"oneOfMany\"/>   \n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.getRuleApplication(),
				is(RuleApplicationEnum.oneOfMany));
	}

	@Test
	public void testCreateConfigFromFile_shouldNotWriteComments_1()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <writeComments value=\"false\"/>   \n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.isWriteComments(), is(false));
	}

	@Test
	public void testCreateConfigFromFile_shouldNotWriteComments_2()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.isWriteComments(), is(false));
	}

	@Test
	public void testCreateConfigFromFile_shouldSetRandomInit() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <writeComments value=\"true\"/>   \n"
				+ "   <randomInit value=\"0\"/>\n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat((int) config.getRandomInit(), is(0));
	}

	@Test
	public void testCreateConfigFromFile_shouldWriteComments() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <configuration>\n"
				+ "   <writeComments value=\"true\"/>   \n"
				+ " </configuration>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"deleteRule\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		assertThat(config.isWriteComments(), is(true));
	}

	@Test
	public void testCreateConfigFromFile_swapDefaultNeagtionMR()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"swapDefaultNegation\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0),
				is(SwitchDefaultNegationModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_swapStrongNegationMR()
			throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"swapStrongNegation\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0),
				is(SwapStrongNegationModificationRule.class));
	}

	@Test
	public void testCreateConfigFromFile_swapTermsMR() throws Exception {
		String xmlCode = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
				+ "<aspmodificator>\n"
				+ " <modificationRules>\n"
				+ "   <modificationRule name=\"rule2\" type=\"swapTerms\" min=\"1\" max=\"1\" />\n"
				+ " </modificationRules>\n" + "</aspmodificator>";
		File file = createTestFile(xmlCode);

		AspModificationConfig config = AspModificationConfig
				.createConfigFromFile(file.getAbsolutePath());

		assertThat(config, notNullValue());

		List<IModificationRule> modRules = config.getModificationRules();

		assertThat(modRules, notNullValue());
		assertThat(modRules.size(), is(1));
		assertThat(modRules.get(0), is(SwapTermModificationRule.class));
	}

}
